import QtQuick 1.1
import com.nokia.meego 1.0

Rectangle {
    id: root
    height: 85 + recentSitesList.height + hideRecent.height
    property int loadProgress: webViewport.child.loadProgress
    property alias url: addressField.text
    property alias inputFocus: addressField.activeFocus
    color: theme.inverted ? "black" : "white"
    property bool showRecent: false
    property bool setBackspace: false

    function selectAll() {
        selectAllTimer.start()
    }

    function focusAndSelectAll() {
        addressField.forceActiveFocus()
        root.selectAll()
    }

    function fillRecentFromDatabase(value) {
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                var result = tx.executeSql('SELECT url, title, icon FROM history where url like (?) order by date desc limit 5',["%" + value + "%"])
                recentListModel.clear()
                for (var i=0; i < result.rows.length; i++) {
                    recentListModel.insert(0, {"url": result.rows.item(i).url,
                                     "title": result.rows.item(i).title,
                                     "icon": result.rows.item(i).icon})
                }
            }
        );
    }

    function setUrl(value) {
        if (!root.setBackspace) {
            var oldLength = addressField.text.length
            if (oldLength > 0) {
                addressField.text = value
                addressField.cursorPosition = 0
                addressField.select(addressField.text.length, oldLength)
            }
        }
        else {
            root.setBackspace = false
        }
    }

    function hideRecentList() {
        root.showRecent = false
        recentSitesList.height = 0
    }

    Connections {
        target: viewportMouse
        onReleased: {
            if (root.visible) {
                var mapped = root.mapFromItem(mainScope, mouse.x, mouse.y)
                if (mapped.x > -1 && mapped.x < root.width && mapped.y > -1 && mapped.y < root.height) {
                    root.focusAndSelectAll()
                }
            }
        }
    }

    Timer {
        id: selectAllTimer
        interval: 500
        repeat: false
        onTriggered: {
            addressField.cursorPosition = addressField.text.length
            addressField.platformOpenSoftwareInputPanel()
            addressField.selectAll()
        }
    }

    Connections {
        target: webViewport.child
        onUrlChanged: {
            addressField.text = webViewport.child.url
            addressField.cursorPosition = 0
        }
    }

    Rectangle {
        id: progress
        anchors.top: parent.top
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        width: parent.width / 100 * root.loadProgress
        color: "cyan"
        opacity: 0.6
        visible: (root.loadProgress > 0) ? (root.loadProgress < 100 ? true : false) : false
        smooth: true
    }

    Rectangle {
        color: theme.inverted ? "white" : "black"
        width: parent.width
        height: 1
        anchors.bottom: root.bottom
        anchors.bottomMargin: 1
    }

    Rectangle {
        color: theme.inverted ? "black" : "white"
        width: parent.width
        height: 1
        anchors.bottom: root.bottom
    }

    Text {
        id: title
        height: 18
        anchors.left: parent.left
        anchors.leftMargin: 10
        anchors.right: parent.right
        anchors.rightMargin: 10
        anchors.top: root.top
        horizontalAlignment: (paintedWidth > parent.width) ? Text.AlignLeft : Text.AlignHCenter
        font.pixelSize: height
        color: theme.inverted ? "white" : "black"
        text: webViewport.child.title
    }

    FoxTextField {
        id: addressField
        anchors.top: parent.top
        anchors.topMargin: 25
        anchors.horizontalCenter: parent.horizontalCenter
        width: root.width - 10
        inputMethodHints: Qt.ImhNoPredictiveText || Qt.ImhNoAutoUppercase

        Keys.onReturnPressed: {
            hideRecentList()
            mainPage.load(addressField.text)
            addressField.platformCloseSoftwareInputPanel()
        }

        onBackspacePressed: {
            root.setBackspace = true
        }

        onTextChanged: {
            if (addressField.activeFocus) {
                fillRecentFromDatabase(text)
                root.showRecent = (recentListModel.count > 0)
                if (recentListModel.count > 0) {
                    var url0 = recentListModel.get(0).url
                    if (url0.search(text) == 0) {
                        setUrl(url0)
                    }
                }
                if (recentListModel.count > 4) {
                    recentSitesList.height = 200
                }
                else {
                    recentSitesList.height = (50 * recentListModel.count)
                }
            }
        }
    }

    ListView {
        id: recentSitesList
        anchors.top: addressField.bottom
        anchors.topMargin: 10
        anchors.left: parent.left
        anchors.right: parent.right
        height: 0
        clip: true
        visible: root.showRecent
        model: recentListModel
        delegate: Item {
            id: itemDelegate
            width: parent.width
            height: 50
            Rectangle {
                anchors.fill: parent
                color: mArea.pressed ? "#808080" : "transparent"
            }
            Image {
                id: siteIcon
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: parent.left
                anchors.leftMargin: 3
                source: model.icon ? (testIcon.status == Image.Error ? QmlHelperTools.getFaviconFromUrl(model.url) : model.icon) : QmlHelperTools.getFaviconFromUrl(model.url)
                width: 20
                height: 20
                smooth: true
                cache: true
            }
            Image {
                id: testIcon
                visible: false
                source: model.icon
            }
            Text {
                id: siteLabel
                anchors.top: parent.top
                anchors.topMargin: 3
                anchors.left: siteIcon.right
                anchors.leftMargin: 3
                text: model.title
                font.pixelSize: 18
                color: theme.inverted ? "white" : "black"
            }
            Text {
                id: siteUrl
                anchors.top: siteLabel.bottom
                anchors.topMargin: 1
                anchors.left: siteIcon.right
                anchors.leftMargin: 3
                text: model.url
                font.pixelSize: 18
                color: theme.inverted ? "white" : "black"
            }
            MouseArea {
                id: mArea
                anchors.fill: itemDelegate
                onClicked: {
                    hideRecentList()
                    mainPage.load(model.url)
                    addressField.text = model.url
                }
            }
        }
    }

    ScrollDecorator {
        flickableItem: recentSitesList
    }

    ListModel {
        id: recentListModel
    }

    Rectangle {
        id: hideRecent
        anchors.top: recentSitesList.bottom
        width: parent.width
        height: visible ? 30 : 0
        visible: recentSitesList.visible
        color: theme.inverted ? "black" : "white"
        Rectangle {
            width: parent.width / 3 * 2
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.top: parent.top
            anchors.topMargin: 10
            height: 1
            color: theme.inverted? "white" : "black"
        }
        Rectangle {
            width: parent.width / 3 * 2
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.top: parent.top
            anchors.topMargin: 20
            height: 1
            color: theme.inverted? "white" : "black"
        }
        Rectangle {
            anchors.bottom: parent.bottom
            width: parent.width
            height: 1
            color: theme.inverted? "white" : "black"
        }
        MouseArea {
            anchors.fill:parent
            onClicked: {
                hideRecentList()
            }
        }
    }
} 
