import QtQuick 1.1
import com.nokia.meego 1.0

Item {
    id: root
    property bool simple: true
    property int offsetX: 0
    property int offsetY: 0
    property int newAnchorX: 0
    property int newAnchorY: 0
    property int totalOffsetX: 0
    property int totalOffsetY: 0

    property bool active: true
    signal tap

    Rectangle {
        id: anchorLine
        width: parent.width
        height: 1
        y: touchLine.y - root.newAnchorY
        color: theme.inverted ? "white" : "black"
        opacity: 0.5
        visible: root.simple
    }

    Rectangle {
        id: touchLine
        width: parent.width
        height: 1
        y: testLink.y + (testLink.height / 2) + root.newAnchorY
        color: "red"
        opacity: 0.5
        visible: root.simple
    }

    Rectangle {
        id: topA
        anchors.top: testLink.top
        width: parent.width
        height: 1
        color: "red"
        opacity: 0.2
        visible: root.simple
    }

    Rectangle {
        id: topB
        anchors.bottom: testLink.bottom
        width: parent.width
        height: 1
        color: "red"
        opacity: 0.2
        visible: root.simple
    }

    Text {
        id: testLink
        anchors.centerIn: parent
        font.pixelSize: 16
        text: qsTrId("meefox_test_vo_title")
        font.underline: true
        color: theme.inverted ? "white" : "black"
        visible: root.simple
    }

    Rectangle {
        id: anchorLineX
        width: parent.width
        height: 1
        x: 0
        y: touchLineX.y - root.newAnchorY
        color: theme.inverted ? "white" : "black"
        visible: !root.simple
    }

    Rectangle {
        id: anchorLineY
        width: 1
        height: parent.height
        x: touchLineY.x - root.newAnchorX
        y: 0
        color: theme.inverted ? "white" : "black"
        visible: !root.simple
    }

    Rectangle {
        id: touchLineX
        width: parent.width
        height: 1
        x: 0
        y: innerC.y + (innerC.height / 2) + root.newAnchorY
        color: "red"
        visible: !root.simple
    }

    Rectangle {
        id: touchLineY
        width: 1
        height: parent.height
        x: innerC.x + (innerC.width / 2) + root.newAnchorY
        y: 0
        color: "red"
        visible: !root.simple
    }

    Rectangle {
        id: outterC
        anchors.centerIn: parent
        width: Math.min(parent.height, parent.width) - 20
        height: width
        radius: width / 2
        border.width: 1
        border.color: "green"
        color: "transparent"
        opacity: 0.4
        smooth: true
        visible: !root.simple
    }

    Rectangle {
        id: innerC
        anchors.centerIn: parent
        width: 50
        height: width
        radius: width / 2
        border.width: 1
        border.color: "yellow"
        color: "transparent"
        smooth: true
        visible: !root.simple
    }

    Text {
        id: testLinkDynamic
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.top: innerC.bottom
        anchors.topMargin: root.active ? 100 : 10
        font.pixelSize: 20
        text: qsTrId("meefox_test_do_title")
        color: theme.inverted ? "white" : "black"
        visible: !root.simple
    }

    MouseArea {
        anchors.fill: parent
        onPressed: {
            root.tap()
            if (root.simple) {
                touchLine.y = mouse.y
                root.offsetY = mouse.y - testLink.y - (testLink.height / 2)
                root.totalOffsetY += root.offsetY
            }
            else {
                touchLineX.y = mouse.y
                touchLineY.x = mouse.x
                root.offsetY = mouse.y - innerC.y - (innerC.height / 2)
                root.totalOffsetY += root.offsetY
                root.offsetX = mouse.x - innerC.x - (innerC.width / 2)
                root.totalOffsetX += root.offsetX
            }
        }
    }
} 
