import QtQuick 1.1
import com.nokia.meego 1.0

Page {
    id: root
    signal bookmarksChanged()
    property bool addMode: false

    tools: ToolBarLayout {
        ToolIcon { iconId: "toolbar-back"; onClicked: pageStack.pop() }
        ToolIcon { iconId: "toolbar-favorite-" + ( root.addMode ? "un" : "" ) + "mark"; onClicked: {
                newSiteUrl.text = ""
                newSiteTitle.text = ""
                showOnStart.checked = false
                selectGroup.text = qsTrId("meefox_dialog_group")
                root.addMode = !root.addMode
            }
        }
    }

    PageHeader {
        id: header
        anchors.top: root.top
        width: root.width
        titleText: qsTrId("meefox_bookmarks_title")
        iconSource: mainPage.iconset + "/bookmarks.png"
    }

    FoxTextField {
        id: filterArea
        anchors.left: parent.left
        anchors.leftMargin: 10
        anchors.top: header.bottom
        anchors.topMargin: 15
        anchors.right: parent.right
        anchors.rightMargin: 10
        inputMethodHints: Qt.ImhNoPredictiveText
        visible: !root.addMode
        Keys.onReturnPressed: {
            filterModel(text)
        }
    }

    Item {
        id: addArea
        anchors.left: parent.left
        anchors.leftMargin: 10
        anchors.bottom: parent.bottom
        anchors.right: parent.right
        anchors.rightMargin: 10
        height: urlLabel.height +
                newSiteUrl.height +
                titleLabel.height +
                newSiteTitle.height + 
                selectGroup.height + 5 +
                addButton.height + 10
        visible: root.addMode

        Text {
            id: urlLabel
            anchors.top: parent.top
            anchors.left: parent.left
            anchors.right: parent.right
            font.pixelSize: 26
            text: qsTrId("meefox_bookmarks_site_url")
            color: theme.inverted ? "white" : "black"
        }

        FoxTextField {
            id: newSiteUrl
            function fixSchemeUrl() {
                if (text.search("//") == -1) {
                    text = "http://" + text
                }
                return text
            }
            anchors.top: urlLabel.bottom
            anchors.left: parent.left
            anchors.right: parent.right
            inputMethodHints: Qt.ImhNoPredictiveText || Qt.ImhNoAutoUppercase
        }

        Text {
            id: titleLabel
            anchors.top: newSiteUrl.bottom
            anchors.left: parent.left
            anchors.right: parent.right
            font.pixelSize: 26
            text: qsTrId("meefox_bookmarks_site_title")
            color: theme.inverted ? "white" : "black"
        }

        FoxTextField {
            id: newSiteTitle
            anchors.top: titleLabel.bottom
            anchors.left: parent.left
            anchors.right: parent.right
            inputMethodHints: Qt.ImhNoPredictiveText || Qt.ImhNoAutoUppercase
        }

        Button {
            id: selectGroup
            anchors.top: newSiteTitle.bottom
            anchors.topMargin: 5
            anchors.left: parent.left
            anchors.right: parent.right
            height: 50
            text: qsTrId("meefox_dialog_group")
            function callback(index) {
                text = dialog.selectedName
                dialog.accepted.disconnect(selectGroup.callback)
            }
            onClicked: {
                dialog.model = groupsListModel
                dialog.open()
                dialog.accepted.connect(selectGroup.callback)
            }
        }

        Switch {
            id: showOnStart
            anchors.left: parent.left
            anchors.verticalCenter: addButton.verticalCenter
        }

        Text {
            text: qsTrId("meefox_bookmarks_pin")
            font.pixelSize: 26
            anchors.verticalCenter: showOnStart.verticalCenter
            anchors.left: showOnStart.right
            anchors.leftMargin: 10
            color: theme.inverted ? "white" : "black"
        }

        Button {
            id: addButton
            height: 50
            width: parent.width / 3
            anchors.top: selectGroup.bottom
            anchors.topMargin: 5
            anchors.right: parent.right
            text: qsTrId("meefox_bookmarks_add")
            enabled: newSiteTitle.text.length > 0 && newSiteUrl.text.length > 0
            onClicked: {
                mainPage.addBookmark(newSiteUrl.fixSchemeUrl(), newSiteTitle.text, selectGroup.text, showOnStart.checked ? 1 : 0)
                root.addMode = false
                filterArea.text = ""
                bookmarksChanged()
                fillModelFromDatabase()
                fillGroupsModel()
            }
        }

        Rectangle {
            anchors.top: parent.top
            width: parent.width
            height: 1
            color: theme.inverted ? "white" : "black"
        }
    }

    Component.onCompleted: {
        print("Bookmarks ready!")
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                tx.executeSql('CREATE TABLE IF NOT EXISTS bookmarks (url TEXT, title TEXT, icon TEXT, category TEXT, type INTEGER)')
            }
        );
        fillModelFromDatabase()
        fillGroupsModel()
    }

    function updateBookmark(url, title, group, type, oldUrl) {
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                var result = tx.executeSql('update bookmarks set url=(?), title=(?), icon=(?), category=(?), type=(?) where url=(?)',[url, title, QmlHelperTools.getFaviconFromUrl(url), group, type, oldUrl])
                if (result.rowsAffected < 1) {
                    console.log("Error updating bookmark")
                }
            }
        );
    }

    function fillGroupsModel() {
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                var result = tx.executeSql('SELECT distinct category from bookmarks')
                groupsListModel.clear()
                for (var i=0; i < result.rows.length; i++) {
                    groupsListModel.insert(0, {"name": result.rows.item(i).category})
                }
            }
        );
    }

    function fillModelFromDatabase() {
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                var result = tx.executeSql('SELECT * from bookmarks order by category desc')
                bookmarksListModel.clear()
                for (var i=0; i < result.rows.length; i++) {
                    bookmarksListModel.insert(0, {"url": result.rows.item(i).url,
                                                  "title": result.rows.item(i).title,
                                                  "icon": result.rows.item(i).icon,
                                                  "group": result.rows.item(i).category,
                                                  "type": parseInt(result.rows.item(i).type)})
                }
            }
        );
        root.bookmarksChanged()
    }

    function filterModel(value) {
        if (value == "") {
            bookmarksList.model = bookmarksListModel
        }
        else {
            filterListModel.clear()
            var lowerValue = value.toLowerCase()
            for (var i=0; i<bookmarksListModel.count; i++) {
                var item = bookmarksListModel.get(i)
                if (item.title.toLowerCase().search(lowerValue) != -1 ||
                    item.url.toLowerCase().search(lowerValue) != -1) {
                    filterListModel.append(item)
                }
            }
            bookmarksList.model = filterListModel
        }
    }

    function isNewGroup(index) {
        if (index == 0) {
            return true
        }
        else if (index > 0) {
            var prevGroup = bookmarksList.model.get(index - 1).group
            var currGroup = bookmarksList.model.get(index).group
            return prevGroup != currGroup
        }
        return false
    }

    function checkUrl(url) {
        for (var i=0; i<bookmarksListModel.count; i++) {
            if (bookmarksListModel.get(i).url == url) {
                return true
            }
        }
        return false
    }

    ListModel {
        id: groupsListModel
    }

    ListModel {
        id: filterListModel
    }

    ListModel {
        id: bookmarksListModel
    }        

    ListView {
        id: bookmarksList
        clip: true
        anchors.top: filterArea.bottom
        anchors.left: root.left
        anchors.right: root.right
        anchors.bottom: root.addMode ? addArea.top : root.bottom
        anchors.margins: 5
        spacing: 3
        model: bookmarksListModel
        signal hideAll()
        delegate: Item {
            id: historyDelegate
            width: parent.width
            height: showControls ? (siteTitleEdit.height + siteUrlEdit.height + editGroup.height + saveBookmark.height + 30) : (groupSeparator.height + siteTitle.height + siteUrl.height + 15)
            property bool showControls: false

            Connections {
                target: bookmarksList
                onHideAll: {
                    showControls = false
                }
            }

            Rectangle {
                id: bg
                anchors.fill: parent
                anchors.topMargin: groupSeparator.height + (groupSeparator.visible ? 5 : 0)
                anchors.margins: 2
                color: mArea.pressed ? ( theme.inverted ? "#606060" : "#a0a0a0" ) : ( theme.inverted ? ( index%2==1 ? "#202020" : "transparent" ) : ( index%2==1 ? "#d0d0d0" : "transparent" ) )
            }

            Rectangle {
                anchors.top: parent.top
                anchors.bottom: siteTitle.top
                width: parent.width
                visible: groupSeparator.visible
                color: theme.inverted ? "white" : "black"
            }

            Text {
                id: groupSeparator
                anchors.left: parent.left
                anchors.leftMargin: 2
                anchors.top: parent.top
                font.pixelSize: 18
                text: visible ? model.group : ""
                visible: !showControls && isNewGroup(index)
                height: visible ? 20 : 0
                color: theme.inverted ? "black" : "white"
            }

            Image {
                id: siteIcon
                source: model.icon ? (testIcon.status == Image.Error ? QmlHelperTools.getFaviconFromUrl(model.url) : model.icon) : QmlHelperTools.getFaviconFromUrl(model.url)
                width: 20
                height: 20
                smooth: true
                anchors.left: parent.left
                anchors.leftMargin: 5
                anchors.verticalCenter: showControls ? siteTitleEdit.verticalCenter : siteTitle.verticalCenter
                cache: true
            }

            Image {
                id: testIcon
                visible: false
                source: model.icon
            }

            Button {
                id: editBookmark
                width: 50
                height: 50
                anchors.right: parent.right
                anchors.rightMargin: 5
                anchors.top: siteTitle.top
                anchors.topMargin: 5
                visible: !showControls
                iconSource: mainPage.iconset + "/edit.png"
                onClicked: {
                    bookmarksList.hideAll()
                    showControls = true
                }
            }

            Text {
                id: siteTitle
                anchors.top: groupSeparator.visible ? groupSeparator.bottom : parent.top
                anchors.topMargin: 5
                anchors.left: siteIcon.right
                anchors.leftMargin: 10
                anchors.right: editBookmark.left
                anchors.rightMargin: 5
                text: model.title
                font.pixelSize: 20
                elide: Text.ElideRight
                clip: true
                visible: !showControls
                color: theme.inverted ? "white" : "black"
            }

            FoxTextField {
                id: siteTitleEdit
                anchors.top:parent.top
                anchors.topMargin: 5
                anchors.left: siteIcon.right
                anchors.leftMargin: 10
                anchors.right: parent.right
                anchors.rightMargin: 10
                text: model.title
                visible: showControls
            }

            Text {
                id: siteUrl
                text: model.url
                anchors.top: siteTitle.bottom
                anchors.topMargin: 5
                anchors.left: siteIcon.right
                anchors.leftMargin: 10
                anchors.right: editBookmark.left
                anchors.rightMargin: 5
                font.pixelSize: 20
                font.underline: true
                elide: Text.ElideRight
                clip: true
                visible: !showControls
                color: theme.inverted ? "white" : "black"
            }

            FoxTextField {
                id: siteUrlEdit
                function fixSchemeUrl() {
                    if (text.search("//") == -1) {
                        text = "http://" + text
                    }
                    return text
                }
                text: model.url
                anchors.top: siteTitleEdit.bottom
                anchors.topMargin: 5
                anchors.left: siteIcon.right
                anchors.leftMargin: 10
                anchors.right: parent.right
                anchors.rightMargin: 10
                visible: showControls
            }

            Button {
                id: editGroup
                anchors.top: siteUrlEdit.bottom
                anchors.topMargin: 5
                anchors.right: parent.right
                anchors.rightMargin: 10
                width: parent.width / 2 - 10
                height: 50
                text: model.group
                visible: showControls
                function callback(index) {
                    text = dialog.selectedName
                    dialog.accepted.disconnect(editGroup.callback)
                    fillGroupsModel()
                    bookmarksChanged()
                }
                onClicked: {
                    dialog.model = groupsListModel
                    dialog.open()
                    dialog.accepted.connect(editGroup.callback)
                }
            }

            Switch {
                id: pinOnStart
                anchors.left: siteIcon.right
                anchors.leftMargin: 10
                anchors.verticalCenter: editGroup.verticalCenter
                checked: model.type == 1
                visible: showControls
            }

            Text {
                text: qsTrId("meefox_bookmarks_pin_text")
                font.pixelSize: 26
                anchors.verticalCenter: pinOnStart.verticalCenter
                anchors.left: pinOnStart.right
                anchors.leftMargin: 10
                visible: showControls
                color: theme.inverted ? "white" : "black"
            }

            Button {
                id: saveBookmark
                anchors.top: editGroup.bottom
                anchors.topMargin: 5
                anchors.right: deleteBookmark.left
                anchors.rightMargin: 10
                width: 50
                height: 50
                iconSource: mainPage.iconset + "/edit.png"
                visible: showControls
                onClicked: {
                    updateBookmark(siteUrlEdit.fixSchemeUrl(), siteTitleEdit.text, editGroup.text, pinOnStart.checked ? 1 : 0, model.url)
                    showControls = false
                    fillModelFromDatabase()
                    fillGroupsModel()
                    bookmarksChanged()
                }
            }

            Button {
                id: deleteBookmark
                anchors.top: editGroup.bottom
                anchors.topMargin: 5
                anchors.right: parent.right
                anchors.rightMargin: 10
                width: 50
                height: 50
                iconSource: mainPage.iconset + "/remove.png"
                visible: showControls
                onClicked: {
                    removeBookmark(model.url)
                    showControls = false
                    bookmarksChanged()
                }
            }

            MouseArea {
                id: mArea
                anchors.top: parent.top
                anchors.left: parent.left
                anchors.right: editBookmark.left
                anchors.rightMargin: showControls ? parent.width : 10
                anchors.bottom: siteUrl.bottom
                onClicked: {
                    bookmarksList.hideAll()
                    if (!showControls) {
                        mainPage.load(model.url)
                        pageStack.pop()
                    }
                }
                onPressAndHold: {
                    bookmarksList.hideAll()
                    showControls = true
                }
            }
        }
    }

    ScrollDecorator {
        id: scrollDecorator
        flickableItem: bookmarksList
        anchors.rightMargin: -5
    }

    ListSelectionDialog {
        id: dialog
        canAdd: true
        titleText: qsTrId("meefox_dialog_group")
        acceptButtonText: ""
        rejectButtonText: qsTrId("meefox_common_cancel")
    }
}
