import QtQuick 1.1
import com.nokia.meego 1.0
import "file:///usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UI

Dialog {
    id: root
    property string titleText: qsTrId("meefox_dialog_select")
    property alias acceptButtonText: acceptButton.text
    property alias rejectButtonText: rejectButton.text
    property variant selectedItems
    property bool selectMulti: false
    property int selectedIndex: -1
    property alias comboData: comboModel
    platformStyle: QueryDialogStyle {}
    title: Item {
        id: titleField
        width: parent.width
        height: titleLabel.height + root.platformStyle.titleColumnSpacing
        Column {
            id: titleFieldCol
            spacing: root.platformStyle.titleColumnSpacing

            anchors.left:  parent.left
            anchors.right:  parent.right
            anchors.top:  parent.top

            width: root.width

            Item {
                id: titleBarTextField
                height: titleLabel.height
                width: parent.width

                Text {
                    id: titleLabel
                    width: parent.width

                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment:   Text.AlignVCenter

                    font.family: root.platformStyle.titleFontFamily
                    font.pixelSize: root.platformStyle.titleFontPixelSize
                    font.bold:  root.platformStyle.titleFontBold
                    font.capitalization: root.platformStyle.titleFontCapitalization
                    elide: root.platformStyle.titleElideMode
                    color: root.platformStyle.titleTextColor
                    text: root.titleText

                }
            }
        }
    }
    content: Item {
        id: queryContentWrapper

        property int upperBound: visualParent ? visualParent.height - titleField.height - buttonColFiller.height - 64
                                                : root.parent.height - titleField.height - buttonColFiller.height - 64

        height: upperBound
        width: root.width

        Item {
            id: queryContent
            width: parent.width
            height: parent.height

            y: root.platformStyle.contentTopMargin

            ListView {
                id: comboContents
                anchors.fill: parent
                flickableDirection: Flickable.VerticalFlick
                clip: true

                interactive:  contentHeight > queryContent.height
                model: comboModel
                delegate: Item {
                    height: 60
                    width: parent.width
                    Rectangle {
                        anchors.fill: parent
                        anchors.margins: 1
                        color: mArea.pressed ? "white" : (selectedItems[index] ? "lightgray" : "transparent")
                    }
                    Text {
                        anchors.left: parent.left
                        anchors.right: parent.right
                        anchors.margins: 3
                        anchors.verticalCenter: parent.verticalCenter
                        font.pixelSize: model.isGroup ? 30 : 20
                        color: (mArea.pressed || selectedItems[index]) ? "black" : root.platformStyle.titleTextColor
                        text: model.label
                    }
                    MouseArea {
                        id: mArea
                        anchors.fill: parent
                        onClicked: {
                            if (!model.isGroup) {
                                if (selectMulti) {
                                    var arr = selectedItems
                                    arr[index] = !arr[index]
                                    selectedItems = arr
                                    arr = []
                                }
                                else {
                                    selectedIndex = model.id
                                    root.accept()
                                }
                            }
                        }
                    }
                }
            }

            ListModel {
                id: comboModel
            }

            ScrollDecorator {
                id: scrollDecorator
                flickableItem: comboContents
                anchors.rightMargin: - UI.SCROLLDECORATOR_LONG_MARGIN - 10 //ToDo: Don't use a hard-coded gap
            }

        }
    }
    buttons: Item {
        id: buttonColFiller
        width: parent.width
        height: childrenRect.height

        anchors.top: parent.top

        //ugly hack to assure, that we're always evaluating the correct height
        //otherwise the topMargin wouldn't be considered
        Item {id: dummy; anchors.fill:  parent}

        Column {
            id: buttonCol
            anchors.top: parent.top
            anchors.topMargin: root.platformStyle.buttonTopMargin
            spacing: root.platformStyle.buttonsColumnSpacing

            height: (acceptButton.text  == "" ? 0 : acceptButton.height)
                    + (rejectButton.text == "" ? 0 : rejectButton.height)
                    + anchors.buttonTopMargin  + spacing

            anchors.horizontalCenter: parent.horizontalCenter

            Button {
                id: acceptButton
                text: ""
                onClicked: accept()
                visible: text != ""
                __dialogButton: true
                platformStyle: ButtonStyle {inverted: true}
            }
            Button {
                id: rejectButton
                text: ""
                onClicked: reject()
                visible: text != ""
                __dialogButton: true
                platformStyle: ButtonStyle {inverted: true}
            }
        }
    }
} 
 
