import QtQuick 1.1
import com.nokia.meego 1.0

Page {
    id: root
    tools: ToolBarLayout {
        ToolIcon {
            iconId: "toolbar-back"
            onClicked: {
                MozContext.sendObserve("embedui:saveprefs", {})
                pageStack.pop()
                prefsList.model = prefsListModel
            }
        }
    }

    PageHeader {
        id: header
        anchors.top: root.top
        width: root.width
        titleText: qsTrId("meefox_config_title")
        iconSource: mainPage.iconset + "/settings.png"
    }

    Connections {
        target: MozContext
        onRecvObserve: {
            if (message == "embed:allprefs") {
                loadingIndicator.visible = true
                var allprefs = data
                prefsListModel.clear()
                for (var i=0; i<allprefs.length; i++) {
                    prefsListModel.append(allprefs[i])
                }
                loadingIndicator.visible = false
            }
        }
    }

    function prefTypeByValue(value) {
        switch (value) {
            case 32: return qsTrId("meefox_type_string")
            case 64: return qsTrId("meefox_type_integer")
            case 128: return qsTrId("meefox_type_boolean")
        }
    }

    function filterModel(value) {
        if (value == "") {
            prefsList.model = prefsListModel
        }
        else {
            loadingIndicator.visible = true
            filterListModel.clear()
            for (var i=0; i<prefsListModel.count; i++) {
                if (prefsListModel.get(i).name.search(value) != -1) {
                    filterListModel.append(prefsListModel.get(i))
                }
            }
            prefsList.model = filterListModel
            loadingIndicator.visible = false
        }
    }

    ListModel {
        id: prefsListModel
    }

    ListModel {
        id: filterListModel
    }

    FoxTextField {
        id: filterArea
        anchors.left: parent.left
        anchors.leftMargin: 10
        anchors.top: header.bottom
        anchors.topMargin: 15
        anchors.right: parent.right
        anchors.rightMargin: 10
        inputMethodHints: Qt.ImhNoPredictiveText
        Keys.onReturnPressed: {
            filterModel(text)
        }
    }

    Rectangle {
        anchors.bottom: parent.bottom
        width: parent.width
        height: 1
        color: theme.inverted? "white" : "black"
    }

    ListView {
        id: prefsList
        clip: true
        anchors.top: filterArea.bottom
        anchors.topMargin: 5
        anchors.left: root.left
        anchors.right: root.right
        anchors.bottom: root.bottom
        spacing: 0
        model: prefsListModel
        signal hideAll()
        signal filterName(string value)
        delegate: prefsListDelegate
    }

    Component {
        id: prefsListDelegate
        Item {
            id: content
            width: parent.width
            height: 50 + (showMore ? 60 : 0)
            property bool showMore: false

            Connections {
                target: prefsList
                onHideAll: {
                    showMore = false
                    prefValueEdit.platformCloseSoftwareInputPanel()
                }
            }

            Rectangle {
                anchors.fill: parent
                anchors.margins: 2
                color: mArea.pressed ? ( theme.inverted ? "#606060" : "#a0a0a0" ) : ( theme.inverted ? ( index%2==1 ? "#202020" : "transparent" ) : ( index%2==1 ? "#d0d0d0" : "transparent" ) )
            }

            Text {
                id: prefName
                anchors.top: parent.top
                anchors.topMargin: 15
                anchors.left: parent.left
                anchors.leftMargin: 10
                anchors.right: showMore? prefType.left : prefValue.left
                anchors.rightMargin: 10
                text: model.name
                font.pixelSize: 20
                elide: Text.ElideRight
                wrapMode: Text.WrapAnywhere
                color: theme.inverted ? (model.modified ? "#0000ff" : "white") : (model.modified ? "#0000ff" : "black")
            }

            Text {
                id: prefValue
                anchors.bottom: prefName.bottom
                anchors.right: parent.right
                width: 100
                text: model.value
                font.pixelSize: 25
                elide: Text.ElideRight
                visible: !showMore
                color: theme.inverted ? "white" : "black"
            }

            Text {
                id: prefType
                anchors.bottom: prefName.bottom
                anchors.right: parent.right
                width: 100
                font.pixelSize: 20
                text: prefTypeByValue(model.type)
                visible: showMore
                color: theme.inverted ? "white" : "black"
            }

            Text {
                id: boolValue
                anchors.left: prefValueBool.right
                anchors.leftMargin: 15
                anchors.verticalCenter: prefValueBool.verticalCenter
                color: theme.inverted ? "white" : "black"
                visible: prefValueBool.visible
                text: prefValueBool.checked ? "true" : "false"
                font.pixelSize: 25
            }

            MouseArea {
                id: mArea
                anchors.fill: content
                onClicked: {
                    prefsList.hideAll()
                    showMore = true
                }
            }

            FoxTextField {
                id: prefValueEdit
                anchors.top: prefName.bottom
                anchors.topMargin: 10
                anchors.left: parent.left
                anchors.leftMargin: 10
                anchors.right: parent.right
                anchors.rightMargin: 10
                text: model.value
                visible: showMore && model.type != 128
                enabled: visible
                inputMethodHints: (model.type == 64 ? Qt.ImhDigitsOnly : 0)
                Keys.onReturnPressed: {
                    if (model.type == 64) {
                        MozContext.setPref(model.name, parseInt(text))
                    }
                    else {
                        MozContext.setPref(model.name, text)
                    }
                    prefsList.model.setProperty(index, "value", text)
                }
                onActiveFocusChanged: {
                    if (prefValueEdit.activeFocus)
                        prefsList.positionViewAtIndex(index, ListView.Beginning)
                }
            }

            FoxSwitch {
                id: prefValueBool
                anchors.top: prefName.bottom
                anchors.topMargin: 10
                anchors.left: parent.left
                anchors.leftMargin: 10
                visible: showMore && model.type == 128
                onChanged: {
                    MozContext.setPref(model.name, checked)
                    prefValue.text = checked ? "true" : "false"
                }
            }
        }
    }

    ScrollDecorator {
        flickableItem: prefsList
    }
    
    BusyIndicator {
        id: loadingIndicator
        anchors.centerIn: parent
        running: visible
        platformStyle: BusyIndicatorStyle { size: "large" }
    }
} 
