import QtQuick 1.1
import com.nokia.meego 1.0

Page {
    id: root
    tools: ToolBarLayout {
        ToolIcon { iconId: "toolbar-back"; onClicked: pageStack.pop() }
    }

    PageHeader {
        id: header
        anchors.top: root.top
        width: root.width
        titleText: qsTrId("meefox_downloads_title")
        iconSource: mainPage.iconset + "/download.png"
    }

    Connections {
        target: MozContext
        onRecvObserve: {
            if (message == "embed:download") {
                switch (data.msg) {
                    case "dl-list": {
                        loadingIndicator.visible = true
                        downloadsListModel.clear()
                        for (var i=0; i<data.list.length; i++) {
                            downloadsListModel.insert(0, {id: data.list[i].id,
                                                       from: data.list[i].from,
                                                       to: data.list[i].to,
                                                       cur: data.list[i].cur,
                                                       max: data.list[i].max,
                                                       state: data.list[i].state,
                                                       percent: data.list[i].percent,
                                                       speed: 0})
                        }
                        loadingIndicator.visible = false
                        break;
                    }
                    default:
                        break;
                }
            }
        }
    }

    function bytesToSize(bytes) {
        var sizes = [ 'n/a', 'bytes', 'KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB'];
        var i = +Math.floor(Math.log(bytes) / Math.log(1024));
        return  (bytes / Math.pow(1024, i)).toFixed( i ? 1 : 0 ) + ' ' + sizes[ isNaN( bytes ) ? 0 : i+1 ];
    }

    function stateToText(value) {
        switch (value) {
            default: return qsTrId("meefox_download_new")
            case 0: return qsTrId("meefox_download_active")
            case 1: return qsTrId("meefox_download_finished")
            case 2: return qsTrId("meefox_download_failed")
            case 3: return qsTrId("meefox_download_cancelled")
            case 4: return qsTrId("meefox_download_paused")
            case 5: return qsTrId("meefox_download_queued")
            case 6: return qsTrId("meefox_download_blocked")
            case 7: return qsTrId("meefox_download_scanning")
            case 8: return qsTrId("meefox_download_suspicious")
        }
    }

    function stateToColor(value) {
        switch (value) {
            default: return "transparent"
            case 0: return theme.inverted ? "white" : "black"
            case 1: return "green"
            case 2: return "red"
            case 3: return "blue"
            case 4: return "cyan"
            case 5: return "lime"
            case 6: return "red"
            case 7: return "lime"
            case 8: return "red"
        }
    }

    ListModel {
        id: downloadsListModel
    }

    ListView {
        id: downloadsList
        signal hideAll()
        clip: true
        anchors.top: header.bottom
        anchors.topMargin: 1
        anchors.left: root.left
        anchors.right: root.right
        anchors.bottom: root.bottom
        spacing: 10
        model: downloadsListModel
        delegate: Item {
            width: parent.width
            height: content.height
            property bool showControls: false
            //color: showControls ? ( theme.inverted ? "#606060" : "#a0a0a0" ) : ( theme.inverted ? ( index%2==1 ? "#202020" : "transparent" ) : ( index%2==1 ? "#d0d0d0" : "transparent" ) )

            Connections {
                target: MozContext
                onRecvObserve: {
                    if (message == "embed:download") {
                        if (data.id == id) {
                            switch (data.msg) {
                                case "dl-progress": {
                                    downloadsListModel.setProperty(index, "cur", data.cur)
                                    downloadsListModel.setProperty(index, "max", data.max)
                                    downloadsListModel.setProperty(index, "speed", data.speed)
                                    downloadsListModel.setProperty(index, "percent", data.percent)
                                    break;
                                }
                                case "dl-state":
                                case "dl-security": {
                                    downloadsListModel.setProperty(index, "state", data.state)
                                    break;
                                }
                                default: {
                                    downloadsListModel.setProperty(index, "state", data.state)
                                    break;
                                }
                            }
                        }
                    }
                }
            }

            Connections {
                target: downloadsList
                onHideAll: {
                    showControls = false
                }
            }

            Rectangle {
                id: stateRect
                anchors.left: parent.left
                anchors.top: parent.top
                height: parent.height
                width: 10
                color: stateToColor(model.state)
            }

            Rectangle {
                id: progress
                color: stateToColor(model.state)
                opacity: (model.state == 1) ? 0.0 : 0.2
                anchors.top: parent.top
                anchors.bottom: parent.bottom
                anchors.left: stateRect.right
                width: (parent.width - stateRect.width) * (percent / 100)
                smooth: true
            }

            MouseArea {
                id: mouseArea
                anchors.fill: content
                onPressed: {
                    downloadsList.hideAll()
                    showControls = true
                }
                onDoubleClicked: {
                    if (model.state == 1) {
                        QmlHelperTools.openFileBySystem(model.to)
                    }
                }
            }

            Column {
                id: content
                anchors.top: parent.top
                anchors.left: stateRect.right
                anchors.leftMargin: 5
                anchors.right: parent.right

                Text {
                    id: infoText
                    font.pixelSize: 20
                    text: bytesToSize(cur) + " of " + bytesToSize(max) + " | " + percent + "%\n"
                        + stateToText(model.state) + (model.state == 0 ? (" | Speed: " + bytesToSize(speed) + "/s") : "" )
                    color: theme.inverted ? "white" : "black"
                }

                Text {
                    id: fromText
                    width: parent.width
                    font.pixelSize: 20
                    wrapMode: Text.WrapAnywhere
                    text: from
                    color: theme.inverted ? "white" : "black"
                }

                Text {
                    id: toText
                    width: parent.width
                    font.pixelSize: 20
                    wrapMode: Text.WrapAnywhere
                    text: to
                    color: theme.inverted ? "white" : "black"
                }

                Item {
                    width: 1
                    height: controls.visible ? 10 : 0
                }

                Item {
                    id: controls
                    visible: showControls
                    height: visible ? 60 : 0
                    width: parent.width

                    Row {
                        spacing: 15
                        anchors.leftMargin: 15

                        Button {
                            id: pauseResumeButton
                            width: 60
                            height: 60
                            iconSource: mainPage.iconset + "/download-" + (model.state == 0 ? "pause" : "start") + ".png"
                            visible: (model.state == 0 || model.state == 4)
                            onClicked: {
                                MozContext.sendObserve("embedui:download", { msg: (model.state == 0 ? "pauseDownload" : "resumeDownload"), id: id })
                            }
                        }

                        Button {
                            id: stopButton
                            width: 60
                            height: 60
                            iconSource: mainPage.iconset + "/download-stop.png"
                            visible: (model.state == 0 || model.state == 4)
                            onClicked: {
                                MozContext.sendObserve("embedui:download", { msg: "cancelDownload", id: id })
                            }
                        }

                        Button {
                            id: retryButton
                            width: 60
                            height: 60
                            iconSource: mainPage.iconset + "/download-retry.png"
                            visible: (model.state == 2 || model.state == 3)
                            onClicked: {
                                MozContext.sendObserve("embedui:download", { msg: "retryDownload", id: id })
                            }
                        }

                        Button {
                            id: removeButton
                            width: 60
                            height: 60
                            iconSource: mainPage.iconset + "/download-remove.png"
                            visible: (model.state != 0 && model.state != 4)
                            onClicked: {
                                MozContext.sendObserve("embedui:download", { msg: "removeDownload", id: id })
                                downloadsListModel.remove(index)
                            }
                        }
                    }

                    Button {
                        id: copyUrlButton
                        width: 60
                        height: 60
                        anchors.right: parent.right
                        iconSource: mainPage.iconset + "/context-window-url2.png"
                        visible: true
                        onClicked: {
                            QmlHelperTools.clipboard = from
                        }
                    }
                }
            }
        }
    }

    ScrollDecorator {
        flickableItem: downloadsList
    }
    
    BusyIndicator {
        id: loadingIndicator
        anchors.centerIn: parent
        running: visible
        platformStyle: BusyIndicatorStyle { size: "large" }
    }
}
