import QtQuick 1.1
import com.nokia.meego 1.0
import "file:///usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UI

Item {
    id: root

    property string text
    property string iconSource
    signal clicked
    property alias pressed: mouseArea.pressed
    property bool topItem: false
    property bool centerItem: false

    property Style platformStyle: MenuItemStyle{}
    property alias style: root.platformStyle // Deprecated

    width: parent ? parent.width: 0
    height: ( root.platformStyle.height == 0 ) ?
            root.platformStyle.topMargin + menuText.paintedHeight + root.platformStyle.bottomMargin :
            root.platformStyle.topMargin + root.platformStyle.height + root.platformStyle.bottomMargin

    function getBackground() {
        var childs = root.parent.children
        var topIndex = -1
        var bottomIndex = childs.length
        var currentIndex = -1
        for (var i = 0; i < childs.length; i++) {
            if (childs[i].visible) {
                if (topIndex < 0) {
                    topIndex = i
                }
                bottomIndex = i
                if (childs[i] == root) {
                    currentIndex = i
                }
            }
        }

        var suffix = ""
        if (topIndex != bottomIndex) {
            if (currentIndex == topIndex) {
                suffix = "-vertical-top"
            }
            else if (currentIndex == bottomIndex) {
                suffix = "-vertical-bottom"
            }
            else {
                suffix = "-vertical-center"
            }
        }
        
        var bg = "image://theme/meegotouch-list" + (theme.inverted ? "-inverted" : "")+"-background" + (pressed ? "-pressed" : "") + suffix
        return bg
    }

    BorderImage {
        id: backgroundImage
        source: getBackground()
        anchors.fill : root
        border { left: 22; top: 22;
        right: 22; bottom: 22 }
    }

    Image {
        id: menuIcon
        fillMode: Image.PreserveAspectFit
        smooth: true
        source: root.iconSource
        anchors.verticalCenter: parent.verticalCenter
        anchors.left: parent.left
        anchors.leftMargin: 16
    }

    Text {
        id: menuText
        text: parent.text
        elide: Text.ElideRight
        font.family : root.platformStyle.fontFamily
        font.pixelSize : root.platformStyle.fontPixelSize
        font.weight: root.platformStyle.fontWeight
        color: root.platformStyle.textColor

        anchors.topMargin : root.platformStyle.topMargin
        anchors.bottomMargin : root.platformStyle.bottomMargin
        anchors.leftMargin : root.platformStyle.leftMargin
        anchors.rightMargin : root.platformStyle.rightMargin

        anchors.top : root.platformStyle.centered ? undefined : root.top
        anchors.bottom : root.platformStyle.centered ? undefined : root.bottom
        anchors.left : menuIcon.right
        anchors.right : root.right
        anchors.verticalCenter : root.platformStyle.centered ? parent.verticalCenter : undefined
    }

    MouseArea {
        id: mouseArea
        anchors.fill: parent
        onClicked: { if (parent.enabled) parent.clicked();}
    }

    onClicked: if (parent) parent.closeLayout();
}
