import QtQuick 1.1
import com.nokia.meego 1.0
import "file:///usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UI

Dialog {
    id: root
    property string titleText: getTitleByMode(root.mode)
    property alias acceptButtonText: acceptButton.text
    property alias rejectButtonText: rejectButton.text
    property string path: ""
    property string filename: ""
    property int mode: 0
    property variant selectedItems
    platformStyle: QueryDialogStyle {}
    function getTitleByMode(mode) {
        switch (mode) {
            case 0: return qsTrId("meefox_filepicker_openfile")
            case 1: return qsTrId("meefox_filepicker_savefile")
            case 2: return qsTrId("meefox_filepicker_opendir")
            case 3: return qsTrId("meefox_filepicker_select")
            default: return " "
        }
    }
    title: Item {
        id: titleField
        width: parent.width
        height: titleLabel.height + root.platformStyle.titleColumnSpacing
        Column {
            id: titleFieldCol
            spacing: root.platformStyle.titleColumnSpacing

            anchors.left:  parent.left
            anchors.right:  parent.right
            anchors.top:  parent.top

            width: root.width

            Item {
                id: titleBarTextField
                height: titleLabel.height
                width: parent.width

                Text {
                    id: titleLabel
                    width: parent.width

                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment:   Text.AlignVCenter

                    font.family: root.platformStyle.titleFontFamily
                    font.pixelSize: root.platformStyle.titleFontPixelSize
                    font.bold:  root.platformStyle.titleFontBold
                    font.capitalization: root.platformStyle.titleFontCapitalization
                    elide: root.platformStyle.titleElideMode
                    color: root.platformStyle.titleTextColor
                    text: root.titleText

                }
            }
        }
    }
    content: Item {
        id: queryContentWrapper

        property int upperBound: visualParent ? visualParent.height - titleField.height - buttonColFiller.height - 64
                                                : root.parent.height - titleField.height - buttonColFiller.height - 64

        height: upperBound
        width: root.width

        Item {
            id: queryContent
            width: parent.width
            height: parent.height

            y: root.platformStyle.contentTopMargin

            Text {
                id: pathText
                width: parent.width
                horizontalAlignment: Text.AlignHCenter
                font.family: root.platformStyle.messageFontFamily
                font.pixelSize: root.platformStyle.messageFontPixelSize
                color: root.platformStyle.messageTextColor
                wrapMode: Text.WordWrap
                text: root.path
            }

            ListView {
                id: folderContents
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: pathText.bottom
                anchors.topMargin: 10
                anchors.bottom: fileText.visible ? fileText.top : parent.bottom
                anchors.bottomMargin: fileText.visible ? 10 : 0

                flickableDirection: Flickable.VerticalFlick
                clip: true

                interactive:  contentHeight > queryContent.height
                model: QmlHelperTools.getFolderModel(root.path)
                delegate: Item {
                    property bool selected: false
                    height: 60
                    width: parent.width
                    Rectangle {
                        anchors.fill: parent
                        anchors.margins: 1
                        color: mArea.pressed ? "white" : (selected ? "lightgray" : "transparent")
                    }
                    Text {
                        anchors.left: parent.left
                        anchors.right: parent.right
                        anchors.margins: 3
                        anchors.verticalCenter: parent.verticalCenter
                        font.pixelSize: modelData[1] == 0 ? 30 : 20
                        color: mArea.pressed ? "black" : root.platformStyle.titleTextColor
                        text: modelData[0]
                    }
                    MouseArea {
                        id: mArea
                        anchors.fill: parent
                        onClicked: {
                            var item = root.path + "/" + modelData[0]
                            if (modelData[1] == 0) {
                                root.path = QmlHelperTools.getFolderCleanPath(item)
                                var emptyArr = []
                                root.selectedItems = emptyArr
                                folderContents.model = null
                                folderContents.model = QmlHelperTools.getFolderModel(root.path)
                            }
                            else {
                                if (root.mode == 3) {
                                    var arr = selectedItems
                                    var itemIndex = arr.indexOf(item)
                                    if (itemIndex > -1) {
                                        arr.splice(itemIndex, 1)
                                        selected = false
                                    }
                                    else {
                                        arr.push(item)
                                        selected = true
                                    }
                                    selectedItems = arr
                                    arr = []
                                }
                                else if (root.mode == 0) {
                                    arr = [ item ]
                                    selectedItems = arr
                                    arr = []
                                    root.accept()
                                }
                            }
                        }
                    }
                }
            }

            FoxTextField {
                id: fileText
                width: folderContents.width
                anchors.bottom: parent.bottom
                font.family: root.platformStyle.messageFontFamily
                font.pixelSize: root.platformStyle.messageFontPixelSize
                text: root.filename
                visible: root.mode == 1
            }

            ScrollDecorator {
                id: scrollDecorator
                flickableItem: folderContents
                anchors.rightMargin: - UI.SCROLLDECORATOR_LONG_MARGIN - 10 //ToDo: Don't use a hard-coded gap
            }

        }
    }
    buttons: Item {
        id: buttonColFiller
        width: parent.width
        height: childrenRect.height

        anchors.top: parent.top

        //ugly hack to assure, that we're always evaluating the correct height
        //otherwise the topMargin wouldn't be considered
        Item {id: dummy; anchors.fill:  parent}

        Column {
            id: buttonCol
            anchors.top: parent.top
            anchors.topMargin: root.platformStyle.buttonTopMargin
            spacing: root.platformStyle.buttonsColumnSpacing

            height: (acceptButton.text  == "" ? 0 : acceptButton.height)
                    + (rejectButton.text == "" ? 0 : rejectButton.height)
                    + anchors.buttonTopMargin  + spacing

            anchors.horizontalCenter: parent.horizontalCenter

            Button {
                id: acceptButton
                text: ""
                onClicked: accept()
                visible: text != ""
                __dialogButton: true
                platformStyle: ButtonStyle {inverted: true}
            }
            Button {
                id: rejectButton
                text: ""
                onClicked: reject()
                visible: text != ""
                __dialogButton: true
                platformStyle: ButtonStyle {inverted: true}
            }
        }
    }
} 
 
