import QtQuick 1.1
import com.nokia.meego 1.0

Menu {
    id: root

    property variant lastContextInfo: null
    property bool linkUrlAvialable: root.lastContextInfo ? root.lastContextInfo.linkURL.length > 0 : false
    property bool mediaUrlAvialable: root.lastContextInfo ? root.lastContextInfo.mediaURL.length > 0 : false
    property bool selectionInfoAvialable: root.lastContextInfo ? root.lastContextInfo.types.indexOf("content-text") !== -1 : false

    signal startSelectionRequested()
    signal startFindOnPageRequested()

    MenuLayout {
		DynamicMenuItem {
            text: qsTrId("meefox_menu_open_link")
            iconSource: mainPage.iconset + "/context-window-url5.png"
            onClicked: MozContext.newWindow(root.lastContextInfo.linkURL, 0)
            visible: linkUrlAvialable
        }
        DynamicMenuItem {
            text: qsTrId("meefox_menu_save_link")
            iconSource: mainPage.iconset + "/context-window-url3.png"
            onClicked: saveFile(root.lastContextInfo.linkURL)
            visible: linkUrlAvialable
        }
        DynamicMenuItem {
            text: qsTrId("meefox_menu_copy_link")
            iconSource: mainPage.iconset + "/context-clipboard-url.png"
            onClicked: QmlHelperTools.clipboard = root.lastContextInfo.linkURL
            visible: linkUrlAvialable
        }
        DynamicMenuItem {
            text: qsTrId("meefox_menu_open_image")
            iconSource: mainPage.iconset + "/context-window-image5.png"
            onClicked: MozContext.newWindow(root.lastContextInfo.mediaURL, 0)
            visible: mediaUrlAvialable
            topItem: !linkUrlAvialable
        }
        DynamicMenuItem {
            text: qsTrId("meefox_menu_save_image")
            iconSource: mainPage.iconset + "/context-window-image4.png"
            onClicked: saveFile(root.lastContextInfo.mediaURL)
            visible: mediaUrlAvialable
        }
        DynamicMenuItem {
            text: qsTrId("meefox_menu_copy_image")
            iconSource: mainPage.iconset + "/context-clipboard-image2.png"
            onClicked: QmlHelperTools.clipboard = root.lastContextInfo.mediaURL
            visible: mediaUrlAvialable
        }
        DynamicMenuItem {
            text: qsTrId("meefox_menu_select_text")
            iconSource: mainPage.iconset + "/context-select-text.png"
            onClicked: root.startSelectionRequested()
            //visible: selectionInfoAvialable
            topItem: !linkUrlAvialable && !mediaUrlAvialable
        }
        DynamicMenuItem {
            text: qsTrId("meefox_menu_search_text")
            iconSource: mainPage.iconset + "/context-search.png"
            onClicked: root.startFindOnPageRequested()
            centerItem: !linkUrlAvialable && !mediaUrlAvialable// && !selectionInfoAvialable
        }
    }
}
