import QtQuick 1.1
import com.nokia.meego 1.0

Page {
    id: root
    tools: ToolBarLayout {
        ToolIcon { iconId: "toolbar-back"; onClicked: pageStack.pop() }
    }

    PageHeader {
        id: header
        anchors.top: root.top
        width: root.width
        titleText: qsTrId("meefox_history_title")
        iconSource: mainPage.iconset + "/history.png"
    }

    Component.onCompleted: {
        print("History ready!")
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                tx.executeSql('CREATE TABLE IF NOT EXISTS history (url TEXT, title TEXT, icon TEXT, date INTEGER)')
            }
        );
    }

    Connections {
        target: mainPage
        onSaveHistory: {
            var date = new Date()
            date = date.getTime()
            if (url.length > 3 && url.substr(0,6) != "about:") {
                var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
                db.transaction(
                    function(tx) {
                        var result = tx.executeSql('delete from history where url=(?);',[url])
                    }
                );
                db.transaction(
                    function(tx) {
                        var result = tx.executeSql('INSERT INTO history VALUES (?,?,?,?);',[url,title,"",date])
                        if (result.rowsAffected < 1) {
                            console.log("Error inserting url")
                        }
                    }
                );
            }
        }
        onUpdateHistoryFavicon: {
            var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
            db.transaction(
                function(tx) {
                    var result = tx.executeSql('update history set icon=(?) where url=(?);',[favicon, url])
                    if (result.rowsAffected < 1) {
                        console.log("Error inserting icon")
                    }
                }
            );
        }
    }

    function calculateLastWeek(startdate) {
        var date = new Date()
        date.setTime(startdate)
        var delta = 0        
        delta += date.getDay() * 86400
        delta += date.getHours() * 3600
        delta += date.getMinutes() * 60
        delta += date.getSeconds()
        delta *= 1000
        delta += date.getMilliseconds()
        return (startdate - delta)
    }

    function fillModelFromDatabase(startdate) {
        historyList.isLoading = true
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        var date = new Date()
        if (startdate == 0) {
            startdate = date.getTime()
            historyListModel.clear()
        }
        else {
            startdate -= 1
            date.setTime(startdate)
        }
        var enddate = calculateLastWeek(startdate)
        var count = 0
        db.transaction(
            function(tx) {
                var result = tx.executeSql('SELECT count(*) as cnt FROM history where date between (?) and (?)', [enddate, startdate])
                count = result.rows.item(0).cnt
            }
        );
        if (count == 0) {
            var allcount = 0
            db.transaction(
                function(tx) {
                    var result = tx.executeSql('SELECT count(*) as cnt FROM history where date < (?)', [startdate])
                    allcount = result.rows.item(0).cnt
                }
            );
            if (allcount == 0) {
                return
            }
            else {
                while (count == 0) {
                    enddate = calculateLastWeek(enddate - 1)
                    db.transaction(
                        function(tx) {
                            var result = tx.executeSql('SELECT count(*) as cnt FROM history where date between (?) and (?)', [enddate, startdate])
                            count = result.rows.item(0).cnt
                        }
                    );
                }
            }
        }

        db.transaction(
            function(tx) {
                var result = tx.executeSql('SELECT * FROM history where date between (?) and (?) order by date desc', [enddate, startdate])
                for (var i=0; i < result.rows.length; i++) {
                    historyListModel.append({"url": result.rows.item(i).url,
                                     "title": result.rows.item(i).title,
                                     "icon": result.rows.item(i).icon,
                                     "date": parseInt(result.rows.item(i).date)})
                }
            }
        );

        historyList.isLoading = false
    }

    function filterModel(value) {
        if (value == "") {
            historyList.model = historyListModel
        }
        else {
            filterListModel.clear()
            var lowerValue = value.toLowerCase()
            for (var i=0; i<historyListModel.count; i++) {
                var item = historyListModel.get(i)
                if (item.title.toLowerCase().search(lowerValue) != -1 ||
                    item.url.toLowerCase().search(lowerValue) != -1) {
                    filterListModel.append(item)
                }
            }
            historyList.model = filterListModel
        }
    }

    function getTimeString(value) {
        var d = new Date()
        d.setTime(value)
        return Qt.formatDateTime(d, "hh:mm")
    }

    function getDateString(value) {
        var d = new Date()
        d.setTime(value)
        return Qt.formatDateTime(d, "dd MMM")
    }

    function isNewDate(index) {
        if (index == 0) {
            return true
        }
        else if (index > 0) {
            var prevDate = historyList.model.get(index - 1).date
            var d1 = new Date()
            d1.setTime(historyList.model.get(index).date)
            var d2 = new Date()
            d2.setTime(prevDate)
            if (d1.getDate() != d2.getDate()) {
                return true
            }
            else {
                return false
            }
        }
        return false
    }

    function deleteUrl(url) {
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                var result = tx.executeSql('delete from history where url=(?);',[url])
            }
        );
        historyList.deleteUrl(historyMenu.url)
    }

    function deleteAll() {
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                var result = tx.executeSql('drop table history;')
            }
        );
        db.transaction(
            function(tx) {
                tx.executeSql('CREATE TABLE IF NOT EXISTS history (url TEXT, title TEXT, icon TEXT, date INTEGER)')
            }
        );
        historyListModel.clear()
        filterListModel.clear()
        historyList.model = historyListModel
    }

    ListModel {
        id: filterListModel
    }

    ListModel {
        id: historyListModel
    }

    ListView {
        id: historyList
        property bool isLoading: false
        property bool isLocked: false
        signal deleteUrl(string url)
        clip: true
        anchors.top: header.bottom
        anchors.topMargin: 5
        anchors.left: root.left
        anchors.right: root.right
        anchors.rightMargin: 5
        anchors.bottom: root.bottom
        model: historyListModel
        delegate: Item {
            id: historyDelegate
            width: parent.width
            height: 70 + dateSeparator.height

            Connections {
                target: historyList
                onDeleteUrl: {
                    if (url == model.url) {
                        historyList.model.remove(index)
                    }
                }
            }

            Rectangle {
                id: bg
                anchors.fill: parent
                anchors.margins: 2
                color: mArea.pressed ? ( theme.inverted ? "#606060" : "#a0a0a0" ) : ( theme.inverted ? ( index%2==1 ? "#202020" : "transparent" ) : ( index%2==1 ? "#d0d0d0" : "transparent" ) )
            }

            Rectangle {
                anchors.top: parent.top
                anchors.bottom: siteTitle.top
                width: parent.width
                visible: dateSeparator.visible
                color: theme.inverted ? "white" : "black"
            }

            Text {
                id: dateSeparator
                anchors.left: parent.left
                anchors.leftMargin: 10
                anchors.top: parent.top
                font.pixelSize: 18
                text: visible ? getDateString(model.date) : ""
                visible: root.isNewDate(index)
                height: visible ? 20 : 0
                color: theme.inverted ? "black" : "white"
            }

            Image {
                id: siteIcon
                source: model.icon ? (testIcon.status == Image.Error ? QmlHelperTools.getFaviconFromUrl(model.url) : model.icon) : QmlHelperTools.getFaviconFromUrl(model.url)
                width: 20
                height: 20
                smooth: true
                anchors.left: parent.left
                anchors.leftMargin: 10
                anchors.verticalCenter: parent.verticalCenter
                cache: true
            }

            Image {
                id: testIcon
                visible: false
                source: model.icon
            }

            Text {
                id: siteTitle
                anchors.top: dateSeparator.visible ? dateSeparator.bottom : parent.top
                anchors.topMargin: 5
                anchors.left: siteIcon.right
                anchors.leftMargin: 10
                anchors.right: siteDate.left
                anchors.rightMargin: 10
                text: model.title
                font.pixelSize: 20
                elide: Text.ElideRight
                color: theme.inverted ? "white" : "black"
            }

            Text {
                id: siteDate
                anchors.top: dateSeparator.visible ? dateSeparator.bottom : parent.top
                anchors.topMargin: 5
                anchors.right: parent.right
                anchors.rightMargin: 10
                text: getTimeString(model.date)
                font.pixelSize: 20
                color: theme.inverted ? "white" : "black"
            }

            Text {
                id: siteUrl
                text: model.url
                font.underline: true
                anchors.left: siteIcon.right
                anchors.leftMargin: 10
                anchors.top: siteTitle.bottom
                anchors.topMargin: 5
                anchors.right: parent.right
                anchors.rightMargin: 10
                font.pixelSize: 20
                elide: Text.ElideRight
                color: theme.inverted ? "#e0e0ff" : "#2020ff"
            }

            MouseArea {
                id: mArea
                anchors.fill: parent
                onClicked: {
                    mainPage.load(model.url)
                    pageStack.pop()
                }
                onPressAndHold: {
                    historyMenu.url = model.url
                    historyMenu.open()
                }
            }
        }

        ScrollDecorator {
            id: scrollDecorator
            flickableItem: historyList
            anchors.rightMargin: -5
        }

        onMovementStarted: {

        }

        onMovementEnded: {
            
        }

        Connections {
            target: historyList.visibleArea
            onYPositionChanged: {
                if (historyList.flicking)
                    return;

                var contentYPos = historyList.visibleArea.yPosition * Math.max(historyList.height, historyList.contentHeight);

                if (!historyList.isLoading && (historyList.contentHeight < historyList.height || (contentYPos + historyList.height) - historyList.contentHeight > 30)) {
                    var model = historyListModel.get(historyListModel.count - 1)
                    if (model) {
                        var lastdate = model.date
                        fillModelFromDatabase(lastdate)
                    }
                }
            }
        }
    }

    Menu {
        id: historyMenu
        property string url: ""
        MenuLayout {
            DynamicMenuItem {
                text: qsTrId("meefox_history_delete")
                iconSource: mainPage.iconset + "/download-stop.png"
                onClicked: {
                    root.deleteUrl(historyMenu.url)
                }
            }
            DynamicMenuItem {
                text: qsTrId("meefox_history_remove_all")
                iconSource: mainPage.iconset + "/download-remove.png"
                onClicked: {
                    root.deleteAll()
                }
            }
        }
    }
}
