import QtQuick 1.1
import com.nokia.meego 1.0
import "file:///usr/lib/qt4/imports/com/nokia/meego/UIConstants.js" as UI

Dialog {
    id: root
    property string titleText: ""
    property alias acceptButtonText: acceptButton.text
    property alias rejectButtonText: rejectButton.text
    property bool canAdd: false
    property int selectedIndex: -1
    property string selectedName: ""
    property string selectedTitle: ""
    property alias model: listContents.model
    platformStyle: QueryDialogStyle {}
    function tryAddGroup(value) {
        var index = -1
        for (var i=0; i<model.count; i++) {
            if (model.get(i).name == value) {
                index = i
                break;
            }
        }
        if (index == -1) {
            listContents.model.append({"name": value})
            index = listContents.model.count - 1
        }
        selectedIndex = index
        selectedName = value
        accept()
    }
    title: Item {
        id: titleField
        width: parent.width
        height: titleLabel.height + root.platformStyle.titleColumnSpacing
        Column {
            id: titleFieldCol
            spacing: root.platformStyle.titleColumnSpacing

            anchors.left:  parent.left
            anchors.right:  parent.right
            anchors.top:  parent.top

            width: root.width

            Item {
                id: titleBarTextField
                height: titleLabel.height
                width: parent.width

                Text {
                    id: titleLabel
                    width: parent.width

                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment:   Text.AlignVCenter

                    font.family: root.platformStyle.titleFontFamily
                    font.pixelSize: root.platformStyle.titleFontPixelSize
                    font.bold:  root.platformStyle.titleFontBold
                    font.capitalization: root.platformStyle.titleFontCapitalization
                    elide: root.platformStyle.titleElideMode
                    color: root.platformStyle.titleTextColor
                    text: root.titleText

                }
            }
        }
    }
    content: Item {
        id: queryContentWrapper

        property int upperBound: visualParent ? visualParent.height - titleField.height - buttonColFiller.height - 64
                                                : root.parent.height - titleField.height - buttonColFiller.height - 64

        height: upperBound
        width: root.width

        Item {
            id: queryContent
            width: parent.width
            height: parent.height

            y: root.platformStyle.contentTopMargin

            ListView {
                id: listContents
                anchors.left: parent.left
                anchors.right: parent.right
                anchors.top: parent.top
                anchors.topMargin: 10
                anchors.bottom: newText.visible ? newText.top : parent.bottom
                anchors.bottomMargin: newText.visible ? 10 : 0

                flickableDirection: Flickable.VerticalFlick
                clip: true

                //interactive:  contentHeight > queryContent.height
                delegate: Item {
                    height: 60
                    width: parent.width
                    Rectangle {
                        anchors.fill: parent
                        anchors.margins: 1
                        color: mArea.pressed ? "white" : "transparent"
                    }
                    Text {
                        anchors.left: parent.left
                        anchors.right: parent.right
                        anchors.margins: 3
                        anchors.verticalCenter: parent.verticalCenter
                        font.pixelSize: 30
                        color: mArea.pressed ? "black" : root.platformStyle.titleTextColor
                        text: model.title ? model.title : (model.name ? model.name : modelData)
                    }
                    MouseArea {
                        id: mArea
                        anchors.fill: parent
                        onClicked: {
                            root.selectedIndex = index
                            root.selectedName = model.name ? model.name : (model.title ? model.title : modelData)
                            root.selectedTitle = model.title ? model.title : (model.name ? model.name : modelData)
                            accept()
                        }
                    }
                }
            }

            FoxTextField {
                id: newText
                width: listContents.width
                anchors.bottom: parent.bottom
                font.family: root.platformStyle.messageFontFamily
                font.pixelSize: root.platformStyle.messageFontPixelSize
                text: ""
                visible: root.canAdd
                Keys.onReturnPressed: {
                    if (text.length > 0) {
                        root.tryAddGroup(text)
                    }
                }
            }

            ScrollDecorator {
                id: scrollDecorator
                flickableItem: listContents
                anchors.rightMargin: - UI.SCROLLDECORATOR_LONG_MARGIN - 10 //ToDo: Don't use a hard-coded gap
            }

        }
    }
    buttons: Item {
        id: buttonColFiller
        width: parent.width
        height: childrenRect.height

        anchors.top: parent.top

        //ugly hack to assure, that we're always evaluating the correct height
        //otherwise the topMargin wouldn't be considered
        Item {id: dummy; anchors.fill:  parent}

        Column {
            id: buttonCol
            anchors.top: parent.top
            anchors.topMargin: root.platformStyle.buttonTopMargin
            spacing: root.platformStyle.buttonsColumnSpacing

            height: (acceptButton.text  == "" ? 0 : acceptButton.height)
                    + (rejectButton.text == "" ? 0 : rejectButton.height)
                    + anchors.buttonTopMargin  + spacing

            anchors.horizontalCenter: parent.horizontalCenter

            Button {
                id: acceptButton
                text: ""
                onClicked: accept()
                visible: text != ""
                __dialogButton: true
                platformStyle: ButtonStyle {inverted: true}
            }
            Button {
                id: rejectButton
                text: ""
                onClicked: reject()
                visible: text != ""
                __dialogButton: true
                platformStyle: ButtonStyle {inverted: true}
            }
        }
    }
} 
 
