import QtMozilla 1.0
import QtQuick 1.1
import com.nokia.meego 1.0
import com.nokia.extras 1.0

Page {
    id: mainScope
    objectName: "mainScope"
    signal pageTitleChanged(string title)
    signal saveHistory(string url, string title)
    signal updateHistoryFavicon(string url, string favicon)
    property bool firstLoad: true
    property alias viewportChild: webViewport.child
    property alias viewportMouseArea: viewportMouse
    property string iconset: "/opt/meefox/icons/inverted"
    property string iconsetName: "inverted"
    property int touchPointOffsetX: 0
    property int touchPointOffsetY: 0
    property int longtapCancelDistance: 15
    property int longTimerInterval: 800

    Component.onCompleted: {
        var obj = QmlHelperTools.getIcons()
        var iconsetSelected = false
        if (obj.length > 0) {
            for (var i=0; i<obj.length; i++) {
                var set = JSON.parse(obj[i])
                settingsPage.addIconset({title: set.name, name: set.path})
                if (set.path == mainScope.iconset) {
                    mainScope.iconset = set.path
                    mainScope.iconsetName = set.name
                    iconsetSelected = true
                }
            }
            if (!iconsetSelected) {
                var def = JSON.parse(obj[0])
                mainScope.iconset = def.path
                mainScope.iconsetName = def.name
            }
            settingsPage.saveSetting("theme_iconset", mainScope.iconset)
        }
    }

    function load(address) {
        webViewport.child.load(address)
    }

    function saveFile(url) {
        var fileName = url.split("/")
        fileName = fileName[fileName.length - 1]
        filepicker.userRequest = true
        filepicker.mode = 1
        filepicker.path = QmlHelperTools.getStorageLocation(0)
        filepicker.filename = fileName
        filepicker.remoteUrl = url
        filepicker.open()            
    }

    function addBookmark(url, title, group, type) {
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                var result = tx.executeSql('delete from bookmarks where url=(?);',[url])
            }
        );
        db.transaction(
            function(tx) {
                var result = tx.executeSql('INSERT INTO bookmarks VALUES (?,?,?,?,?);',[url,title,QmlHelperTools.getFaviconFromUrl(url),group,type])
                if (result.rowsAffected < 1) {
                    console.log("Error inserting url")
                }
            }
        );
        bookmarksPage.fillModelFromDatabase()
        bookmarksPage.fillGroupsModel()
    }

    function removeBookmark(url) {
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                var result = tx.executeSql('delete from bookmarks where url=(?);',[url])
            }
        );
        bookmarksPage.fillModelFromDatabase()
        bookmarksPage.fillGroupsModel()
    }

    function startSelection(x1, y1) {
        webViewport.child.sendAsyncMessage("Browser:SelectionStart", {
                                            xPos: x1 / webViewport.zoom,
                                            yPos: y1 / webViewport.zoom
                                          })
        webViewport.child.sendAsyncMessage("Browser:SelectionMoveStart", {
                                            change: "start"
                                          })
    }

    function doSelectionStart(x1, y1) {
        webViewport.child.sendAsyncMessage("Browser:SelectionMove", {
                                            change: "start",
                                            start: {
                                                xPos: x1 / webViewport.zoom,
                                                yPos: y1 / webViewport.zoom
                                            }
                                          })
    }

    function doSelectionEnd(x2, y2) {
        webViewport.child.sendAsyncMessage("Browser:SelectionMove", {
                                            change: "end",
                                            end: {
                                                xPos: x2 / webViewport.zoom,
                                                yPos: y2 / webViewport.zoom
                                            }
                                          })
    }

    function cancelSelection() {
        webViewport.child.sendAsyncMessage("Browser:SelectionClose", {
                                            clearSelection: true
                                          })
    }

    function selectionCopy() {
        webViewport.child.sendAsyncMessage("Browser:SelectionCopy", {
                                            xPos: selectionArea.selectionRangeStartX + 2,
                                            yPos: selectionArea.selectionRangeStartY + 2
                                          })
    }

    onHeightChanged: {
        if (test.y + test.height + textinputButtons.height + 30 > mainScope.height) {
            textinputButtons.anchors.bottom = mainScope.bottom
            textinputButtons.anchors.bottomMargin = 10
        }
        else {
            textinputButtons.anchors.bottom = test.bottom
            textinputButtons.anchors.bottomMargin = 0 - textinputButtons.height - 30
        }
    }

    Connections {
        target: MozContext
        onOnInitialized: {
            print("QmlMozContext Initialized");
            MozContext.sendObserve("embedui:setprefs", { prefs :
            [
                { n: "extensions.logging.enabled", v: true},
                { n: "extensions.strictCompatibility", v: false},
                { n: "dom.experimental_forms", v: true},
                { n: "xpinstall.whitelist.add", v: "addons.mozilla.org"},
                { n: "xpinstall.whitelist.add", v: "addons.mozilla.org"},
                { n: "xpinstall.whitelist.add.180", v: "marketplace.firefox.com"},
                { n: "security.alternate_certificate_error_page", v: "certerror"},
                { n: "embedlite.azpc.handle.singletap", v: false},
                { n: "embedlite.azpc.json.singletap", v: true},
                { n: "embedlite.azpc.handle.longtap", v: false},
                { n: "embedlite.azpc.handle.doubletap", v: false},
                { n: "embedlite.azpc.json.doubletap", v: true},
                { n: "embedlite.azpc.json.longtap", v: true},
                { n: "embedlite.azpc.json.viewport", v: true},
                { n: "embedlite.azpc.json.scroll", v: false},
                /*{ n: "browser.ui.touch.left", v: 32},
                { n: "browser.ui.touch.right", v: 32},
                { n: "browser.ui.touch.top", v: 48},
                { n: "browser.ui.touch.bottom", v: 16},*/
                { n: "browser.ui.touch.weight.visited", v: 120},
                { n: "browser.download.folderList", v: 2},
                { n: "browser.download.useDownloadDir", v: false},
                { n: "browser.download.manager.retention", v: 2},
                { n: "browser.helperApps.deleteTempFileOnExit", v: false},
                { n: "browser.download.manager.quitBehavior", v: 1},
                { n: "keyword.enabled", v: true}
            ]});
            MozContext.addObservers([
                "embed:download",
                "embed:prefs",
                "embed:allprefs",
                "clipboard:setdata",
                "clipboard:getdata",
                "embed:logger",
                "embed:search"
            ]);
            MozContext.sendObserve("embedui:logger", { enabled: true })
            MozContext.sendObserve("embedui:search", {msg:"loadxml", uri:"chrome://embedlite/content/bing.xml", confirm: false})
            MozContext.sendObserve("embedui:search", {msg:"loadxml", uri:"chrome://embedlite/content/google.xml", confirm: false})
            MozContext.sendObserve("embedui:search", {msg:"loadxml", uri:"chrome://embedlite/content/yahoo.xml", confirm: false})
            MozContext.sendObserve("embedui:search", {msg:"loadxml", uri:"chrome://embedlite/content/duckduckgo.xml", confirm: false})
            MozContext.sendObserve("embedui:search", {msg:"loadxml", uri:"chrome://embedlite/content/baidu.xml", confirm: false})
            MozContext.sendObserve("embedui:search", {msg:"loadxml", uri:"chrome://embedlite/content/yandex.xml", confirm: false})
        }
        onRecvObserve: {
            switch (message) {
                case "clipboard:setdata": {
                    QmlHelperTools.clipboard = data.data;
                    break
                }
                case "clipboard:getdata": {
                    MozContext.sendObserve("embedui:clipboard", QmlHelperTools.clipboard )
                    break
                }
                case "embed:download": {
                    if (data.msg == "dl-start") {
                        MozContext.sendObserve("embedui:download", { msg: "requestDownloadsList" })
                        pageStack.push(downloadsPage)
                        break
                    }
                }
            }
        }
    }

    QmlMozView {
        id: webViewport
        parentid: createParentID
        objectName: "webViewport"
        property bool movingHorizontally: false
        property bool movingVertically: true
        property variant visibleArea: QtObject {
            property real yPosition: 0
            property real xPosition: 0
            property real widthRatio: 0
            property real heightRatio: 0
        }
        property bool keyboardOpened: false
        property bool textInputElement: false
        property variant textInputRect: null
        property int viewportOffsetX: 0
        property int viewportOffsetY: 0
        property real zoom: 0.0
        property variant viewportData: null

        function scrollTimeout() {
            webViewport.movingHorizontally = false
            webViewport.movingVertically = false
        }
        Timer {
            id: scrollTimer
            interval: 500
            running: false
            repeat: false
            onTriggered: webViewport.scrollTimeout()
        }

        anchors.fill: parent

        Connections {
            target: QGVWindow
            onDisplayEntered: {
                //webViewport.child.suspendView();
                screen.allowedOrientations = Screen.Portrait | Screen.Landscape
            }
            onDisplayExited: {
                //webViewport.child.resumeView();
            }
        }

        Connections {
            target: webViewport.child
            onViewInitialized: {
                print("QmlMozView Initialized");
                webViewport.child.loadFrameScript("chrome://embedlite/content/embedhelper.js");
                webViewport.child.loadFrameScript("chrome://embedlite/content/SelectHelper.js");
                webViewport.child.addMessageListeners([
                    "embed:filepicker",
                    "embed:permissions",
                    "embed:select",
                    "embed:login",
                    "embed:find",
                    "chrome:linkadded",
                    "embed:alert",
                    "embed:confirm",
                    "embed:prompt",
                    "embed:auth",
                    "embed:editableinfo",
                    "embed:viewport",
                    "AZPC:ScrollDOMEvent",
                    "WebApps:PreInstall",
                    "WebApps:PostInstall",
                    "WebApps:Uninstall",
                    "WebApps:Open",
                    "Content:ContextMenu",
                    "Content:SelectionRange",
                    "Content:SelectionCopied",
                    "Content:HandlerShutdown"]);
                webViewport.child.useQmlMouse = true;
                print("QML View Initialized")
                if (startURL.length != 0 && createParentID == 0) {
                    addressBar.url = startURL
                    load(startURL)
                }
                else if (createParentID == 0) {
                    load("about:blank")
                }
            }
            onBgColorChanged: {
                QmlHelperTools.setViewPaletteColor(QGVWindow, webViewport.child.bgcolor);
            }
            onViewAreaChanged: {
                if (selectionStart.visible || viewportMouse.selectionWasVisible)
                    webViewport.child.sendAsyncMessage("Browser:SelectionUpdate", "")
                var r = webViewport.child.contentRect
                var offset = webViewport.child.scrollableOffset
                var s = webViewport.child.scrollableSize
                webViewport.zoom = mainScope.width / r.width
                webViewport.visibleArea.widthRatio = r.width / s.width
                webViewport.visibleArea.heightRatio = r.height / s.height
                webViewport.visibleArea.xPosition = offset.x
                        * webViewport.visibleArea.widthRatio
                        * webViewport.child.resolution
                webViewport.visibleArea.yPosition = offset.y
                        * webViewport.visibleArea.heightRatio
                        * webViewport.child.resolution
                webViewport.movingHorizontally = true
                webViewport.movingVertically = true
                scrollTimer.restart()
            }
            onTitleChanged: {
                pageTitleChanged(webViewport.child.title)
                saveHistory(webViewport.child.url, webViewport.child.title)
            }
            onUrlChanged: {
                if (firstLoad)
                    firstLoad = false
            }
            onLoadingChanged: {
                if (webViewport.child.loading && selectionStart.visible) {
                    cancelSelection()
                }
                if (webViewport.child.loading && searchArea.visible) {
                    searchArea.doSearch("", false, false)
                    searchArea.visible = false
                }
            }
            onHandleSingleTap: {
                if (selectionStart.visible) {
                    retval.message = true
                    selectionArea.hideSelection()
                }
                if (searchArea.visible) {
                    searchArea.doSearch("", false, false)
                    searchArea.visible = false
                }
                if (toolsMenu.forceVisible) {
                    retval.message = true
                    toolsMenu.forceVisible = false
                }
            }
            onImeNotification: {
                webViewport.keyboardOpened = (state == 1)
                if (webViewport.keyboardOpened) {
                    test.update()
                }
                else {
                    test.visible = false
                }
            }
            onRecvAsyncMessage: {
                switch (message) {
                    case "embed:editableinfo": {
                        webViewport.textInputElement = data.v
                        webViewport.textInputRect = data.rect
                        textinputButtons.canCut = data.canCut
                        textinputButtons.canCopy = data.canCopy
                        textinputButtons.canPaste = data.canPaste
                        if (data.v) {
                            test.handleViewportChange(webViewport.child.contentRect.x, webViewport.child.contentRect.y)
                            test.update()
                        }
                        test.visible = webViewport.textInputElement && webViewport.keyboardOpened
                        break
                    }
                    case "chrome:linkadded": {
                        if (data.rel == "shortcut icon") {
                            updateHistoryFavicon(webViewport.child.url, data.href)
                        }
                        break
                    }
                    case "Content:HandlerShutdown": {
                        if (selectionStart.visible || viewportMouse.selectionWasVisible) {
                            selectionStart.visible = false
                            selectionEnd.visible = false
                            viewportMouse.selectionWasVisible = false
                        }
                        break
                    }
                    case "Content:SelectionRange": {
                        if (true) {
                            if (data.updateStart && data.start.xPos && data.start.yPos) {
                                selectionArea.selectionRangeStartX = data.start.xPos
                                selectionArea.selectionRangeStartY = data.start.yPos
                                if (!selectionStartMouse.pressed && !selectionEndMouse.pressed)
                                    selectionStart.handleViewportChange()
                            }
                            if (data.updateEnd && data.end.xPos && data.end.yPos) {
                                selectionArea.selectionRangeEndX = data.end.xPos
                                selectionArea.selectionRangeEndY = data.end.yPos
                                if (!selectionEndMouse.pressed && !selectionStartMouse.pressed)
                                    selectionEnd.handleViewportChange()
                            }
                        }
                        break
                    }
                    case "embed:viewport": {
                        if (selectionStart.visible || viewportMouse.selectionWasVisible)
                            webViewport.child.sendAsyncMessage("Browser:SelectionUpdate", "")
                        if (false) {
                            console.log(JSON.stringify(data))
                        }
                        webViewport.visibleArea.xPosition = data.x
                                * webViewport.visibleArea.widthRatio
                                * webViewport.child.resolution
                        webViewport.visibleArea.yPosition = data.y
                                * webViewport.visibleArea.heightRatio
                                * webViewport.child.resolution
                        if (false) {
                            console.log("viewport lastX: " + webViewport.viewportOffsetX + " newX: " + data.x)
                            console.log("viewport lastY: " + webViewport.viewportOffsetY + " newY: " + data.y)
                        }
                        if (false) {
                            console.log("VP x: " + data.x + " y: " + data.y)
                            console.log("VP m: " + data.resolution.width)
                        }
                        webViewport.viewportData = data
                        if (test.visible || viewportMouse.texteditWasVisible) {
                            test.handleViewportChange(data.x, data.y)
                            test.update()
                        }
                        if (selectionStart.visible || viewportMouse.selectionWasVisible) {
                            selectionStart.handleViewportChange()
                        }
                        if (selectionEnd.visible || viewportMouse.selectionWasVisible) {
                            selectionEnd.handleViewportChange()
                        }
                        webViewport.viewportOffsetX = data.x
                        webViewport.viewportOffsetY = data.y
                        break
                    }
                    case "embed:filepicker": {
                        filepicker.mode = data.mode
                        filepicker.path = QmlHelperTools.getStorageLocation(0)
                        if (data.title) {
                            filepicker.titleText = data.title
                        }
                        if (data.name) {
                            filepicker.filename = data.name
                        }
                        filepicker.userRequest = false
                        filepicker.winid = data.winid
                        filepicker.open()
                        break;
                    }
                    case "embed:alert": {
                        print("onAlert: title:" + data.title + ", msg:" + data.text + " winid:" + data.winid)
                        alertDialog.titleText = data.title
                        alertDialog.message = data.text
                        alertDialog.winid = data.winid
                        alertDialog.open()
                        break;
                    }
                    case "embed:confirm": {
                        print("onConfirm: title:" + data.title + ", data.text:" + data.text)
                        confirmDialog.titleText = data.title
                        confirmDialog.message = data.text
                        confirmDialog.winid = data.winid
                        confirmDialog.open()
                        break;
                    }
                    case "embed:prompt": {
                        print("onPrompt: title:" + data.title + ", msg:" + data.text)
                        promptDialog.titleText = data.title
                        promptDialog.message = data.text
                        promptDialog.prompt = data.defaultValue
                        promptDialog.winid = data.winid
                        promptDialog.open()
                        break;
                    }
                    case "embed:auth": {
                        print("onAuthRequired: title:" + data.title + ", msg:" + data.text + ", winid:" + data.winid)
                        authDialog.titleText = data.title
                        authDialog.message = data.text
                        authDialog.username = data.defaultValue
                        authDialog.password = ""
                        authDialog.winid = data.winid
                        authDialog.open()
                        break;
                    }
                    case "embed:permissions": {
                        print("grant permissions required: title:" + data.title + ", host:" + data.host + ", uid:" + data.id)
                        permissionsDialog.titleText = data.host
                        permissionsDialog.message = qsTrid("meefox_dialog_permission") + " " + data.title
                        permissionsDialog.dontAsk = false
                        permissionsDialog.uid = data.id
                        permissionsDialog.open()
                        break;
                    }
                    case "embed:login": {
                        if (data.name == "password-save") {
                            var login_message = QmlHelperTools.format(qsTrId("meefox_dialog_login_save"), [data.formdata.displayHost])
                        }
                        else if (data.name == "password-change") {
                            var login_message = qsTrId("meefox_dialog_login_change")
                        }
                        if (data.formdata.displayUser) {
                            var login_info = "\n" + QmlHelperTools.format(qsTrId("meefox_dialog_login_name"), [data.formdata.displayUser])
                        }
                        loginManager.titleText = qsTrId("meefox_dialog_login_title")
                        loginManager.message = login_message + login_info
                        loginManager.uid = data.id
                        loginManager.open()
                        break;
                    }
                    case "WebApps:PostInstall": {
                        break;
                    }
                    case "WebApps:Open": {
                        break;
                    }
                    case "WebApps:Uninstall": {
                        break;
                    }
                    case "Content:ContextMenu": {
                        contextMenu.lastContextInfo = data;
                        break;
                    }
                    default:
                        break;
                    }
            }
            onRecvSyncMessage: {
                switch (message) {
                    case "embed:testsyncresponse": {
                        response.message = {
                            val: "response",
                            numval: 0.04
                        }
                        break;
                    }
                    case "embed:select": {
                        comboSelection.selectMulti = data.multiple
                        var selectArr = []
                        comboSelection.comboData.clear()
                        for (var i=0; i<data.selected.length; i++) {
                            selectArr.push(data.selected[i])
                        }
                        comboSelection.selectedItems = selectArr
                        for (var i=0; i<data.listitems.length; i++) {
                            comboSelection.comboData.append(data.listitems[i])
                        }
                        comboSelection.syncLock = true
                        comboSelection.open()
                        while (comboSelection.syncLock) {
                            QmlHelperTools.processEvents()
                        }
                        if (comboSelection.selectMulti) {
                            response.message = { button: comboSelection.selectedItems }
                        }
                        else {
                            response.message = { button: comboSelection.selectedIndex }
                        }
                        break;
                    }
                    case "WebApps:PreInstall": {
                        response.message = { path: "/tmp/webapps" };
                        break;
                    }
                }
            }
        }

        Timer {
            id: longTimer
            interval: mainScope.longTimerInterval
            repeat: false
            property int posX: 0
            property int posY: 0
            property bool longDisabled: false
            property bool longPressed: viewportMouse.pressed && !longTimer.running && !longTimer.longDisabled
            onTriggered: {
                if (!longDisabled) {
                    //addressBar.forceVisible = true
                }
            }
        }

        MouseArea {
            id: viewportMouse
            anchors.fill: parent
            property bool texteditWasVisible: false
            property bool selectionWasVisible: false
            onPressed: {
                webViewport.child.recvMousePress(mouse.x + mainScope.touchPointOffsetX, mouse.y + mainScope.touchPointOffsetY)
                longTimer.longDisabled = false
                longTimer.start()
                longTimer.posX = mouse.x
                longTimer.posY = mouse.y
                selectionWasVisible = selectionStart.visible
                selectionStart.visible = false
                selectionEnd.visible = false
                texteditWasVisible = test.visible
                test.visible = false
            }
            onReleased: {
                webViewport.child.recvMouseRelease(mouse.x + mainScope.touchPointOffsetX, mouse.y + mainScope.touchPointOffsetY)
                longTimer.stop()
                selectionStart.visible = selectionWasVisible
                selectionEnd.visible = selectionWasVisible
                test.visible = texteditWasVisible
            }
            onPositionChanged: {
                if (!longTimer.longPressed) {
                    webViewport.child.recvMouseMove(mouse.x + mainScope.touchPointOffsetX, mouse.y + mainScope.touchPointOffsetY)
                }
                if (longTimer.running && Math.sqrt(Math.pow((longTimer.posX - mouse.x), 2) + Math.pow((longTimer.posY - mouse.y), 2)) > mainScope.longtapCancelDistance) {
                    longTimer.longDisabled = true
                }
            }
        }

        ScrollIndicator {
            id: scrollIndicator
                flickableItem: webViewport
            }
    }

    OverlayItem {
        id: stopRefreshButton
        anchors.top: parent.top
        anchors.topMargin: addressBar.height
        anchors.horizontalCenter: parent.horizontalCenter
        width: parent.width / 3
        height: (parent.height - addressBar.height - toolsMenu.height) / 3
        visible: (longTimer.longPressed || toolsMenu.forceVisible) && !webViewport.textInputElement
        source: mainScope.iconset + "/" + (webViewport.child.loading ? "stop" : "refresh") + ".png"
        active: true
        onClicked: {
            if (webViewport.child.loading) {
                webViewport.child.stop()
            } else {
                webViewport.child.reload()
            }
        }
    }

    OverlayItem {
        id: goBackButton
        anchors.left: parent.left
        anchors.bottom: menuButton.top
        width: parent.width / 3
        height: (parent.height - addressBar.height - toolsMenu.height) / 3
        visible: (longTimer.longPressed || toolsMenu.forceVisible) && !webViewport.textInputElement
        source: mainPage.iconset + "/backward.png"
        active: webViewport.child.canGoBack
        onClicked: {
            webViewport.child.goBack()
        }
    }

    OverlayItem {
        id: goForwardButton
        anchors.right: parent.right
        anchors.bottom: menuButton.top
        width: parent.width / 3
        height: (parent.height - addressBar.height - toolsMenu.height) / 3
        visible: (longTimer.longPressed || toolsMenu.forceVisible) && !webViewport.textInputElement
        source: mainPage.iconset + "/forward.png"
        active: webViewport.child.canGoForward
        onClicked: {
            webViewport.child.goForward()
        }
    }

    OverlayItem {
        id: menuButton
        anchors.bottom: parent.bottom
        anchors.bottomMargin: toolsMenu.height
        anchors.horizontalCenter: parent.horizontalCenter
        width: parent.width / 3
        height: (parent.height - addressBar.height - toolsMenu.height) / 3
        visible: (longTimer.longPressed || toolsMenu.forceVisible) && !webViewport.textInputElement
        source: mainPage.iconset + "/menu.png"
        active: true
        onClicked: {
            contextMenu.open()
        }
    }

    MouseArea {
        id: selectionArea
        x: Math.min(selectionStart.x, selectionEnd.x) + selectionArea.touchOffsetX
        y: Math.min(selectionStart.y, selectionEnd.y) + selectionArea.touchOffsetY
        width: Math.abs(selectionStart.x - selectionEnd.x)
        height: Math.abs(selectionStart.y - selectionEnd.y)
        property int touchOffsetX: 33
        property int touchOffsetY: 10
        property int verticalTreshold: 20
        property int selectionRangeStartX: 0
        property int selectionRangeStartY: 0
        property int selectionRangeEndX: 0
        property int selectionRangeEndY: 0
        enabled: selectionStart.visible
        function hideSelection() {
            selectionStart.visible = false
            selectionEnd.visible = false
            viewportMouse.selectionWasVisible = false
        }
        function copySelection() {
            selectionCopy()
            cancelSelection()
        }
        function updateSelection() {
            if (selectionStart.isStartHandle) {
                doSelectionStart(selectionStart.x + selectionArea.touchOffsetX, selectionStart.y + selectionArea.touchOffsetY)
                doSelectionEnd(selectionEnd.x + selectionArea.touchOffsetX, selectionEnd.y + selectionArea.touchOffsetY)
            }
            else {
                doSelectionStart(selectionEnd.x + selectionArea.touchOffsetX, selectionEnd.y + selectionArea.touchOffsetY)
                doSelectionEnd(selectionStart.x + selectionArea.touchOffsetX, selectionStart.y + selectionArea.touchOffsetY)
            }            
        }
        onClicked: {
            selectionArea.copySelection()
        }
    }

    Rectangle {
        id: test
        color: "transparent"
        smooth: true
        visible: false
        function handleViewportChange(posX, posY) {
            if (webViewport.textInputRect) {
                test.x = (webViewport.textInputRect.x - posX) * webViewport.zoom
                test.y = (webViewport.textInputRect.y - posY) * webViewport.zoom
                test.width = (webViewport.textInputRect.w * webViewport.zoom)
                test.height = (webViewport.textInputRect.h * webViewport.zoom)
            }
        }
        function update() {
            if (webViewport.textInputRect) {
                if (test.y - textinputButtons.height - 20 > 0) {
                    textinputButtons.anchors.bottom = test.top
                    textinputButtons.anchors.bottomMargin = 10
                }
                else if (test.y + test.height + textinputButtons.height + 20 < mainScope.height) {
                    textinputButtons.anchors.bottom = test.bottom
                    textinputButtons.anchors.bottomMargin = 0 - textinputButtons.height - 10
                }
                else {
                    textinputButtons.anchors.bottom = mainScope.bottom
                    textinputButtons.anchors.bottomMargin = 10
                }
                if ((test.x + test.width) > 0 && test.x < mainScope.width) {
                    textinputButtons.x = (mainScope.width - textinputButtons.width) / 2
                }
                else {
                    textinputButtons.x = test.x
                }
            }
        }
    }

    Image {
        id: selectionStart
        visible: false
        smooth: true
        source: "image://theme/meegotouch-text-selection-handle"
        property int initX: 0
        property int initY: 0
        x:0
        y:0
        property bool isStartHandle: (selectionStart.y < selectionEnd.y ||
                                     (selectionStart.x < selectionEnd.x && Math.abs(selectionStart.y - selectionEnd.y) < selectionArea.verticalTreshold * webViewport.zoom))
        function handleViewportChange() {
            selectionStart.x = (selectionArea.selectionRangeStartX) * webViewport.zoom - selectionArea.touchOffsetX
            selectionStart.y = (selectionArea.selectionRangeStartY) * webViewport.zoom - selectionArea.touchOffsetY
        }
        MouseArea {
            id: selectionStartMouse
            anchors.fill: parent
            onPositionChanged: {
                var mapped = mapToItem(mainScope, mouse.x, mouse.y)
                selectionStart.x = mapped.x - selectionArea.touchOffsetX
                selectionStart.y = mapped.y - 80
                if (test.visible) {
                    if (selectionStart.x + selectionArea.touchOffsetX < test.x)
                        selectionStart.x = test.x - selectionArea.touchOffsetX
                    if (selectionStart.y < test.y)
                        selectionStart.y = test.y
                    if (selectionStart.x + selectionArea.touchOffsetX > test.x + test.width)
                        selectionStart.x = test.x + test.width - selectionArea.touchOffsetX
                    if (selectionStart.y > test.y + test.height)
                        selectionStart.y = test.y + test.height
                }
                else {
                    if (selectionStart.x + selectionArea.touchOffsetX < 0)
                        selectionStart.x = 0
                    if (selectionStart.y < 0)
                        selectionStart.y = 0
                    if (selectionStart.x > mainScope.width)
                        selectionStart.x = mainScope.width - selectionArea.touchOffsetX
                    if (selectionStart.y > mainScope.height)
                        selectionStart.y = mainScope.height - selectionStart.height
                }
                selectionArea.updateSelection()
            }
        }
    }

    Image {
        id: selectionEnd
        visible: false
        smooth: true
        source: "image://theme/meegotouch-text-selection-handle"
        property int initX: 0
        property int initY: 0
        x:0
        y:0
        function handleViewportChange() {
            selectionEnd.x = (selectionArea.selectionRangeEndX) * webViewport.zoom - selectionArea.touchOffsetX
            selectionEnd.y = (selectionArea.selectionRangeEndY) * webViewport.zoom - selectionArea.touchOffsetY
        }
        MouseArea {
            id: selectionEndMouse
            anchors.fill: parent
            onPositionChanged: {
                var mapped = mapToItem(mainScope, mouse.x, mouse.y)
                selectionEnd.x = mapped.x - selectionArea.touchOffsetX
                selectionEnd.y = mapped.y - 80
                if (test.visible) {
                    if (selectionEnd.x < test.x)
                        selectionEnd.x = test.x - selectionArea.touchOffsetX
                    if (selectionEnd.y < test.y)
                        selectionEnd.y = test.y
                    if (selectionEnd.x + selectionArea.touchOffsetX > test.x + test.width)
                        selectionEnd.x = test.x + test.width - selectionArea.touchOffsetX
                    if (selectionEnd.y > test.y + test.height)
                        selectionEnd.y = test.y + test.height
                }
                else {
                    if (selectionEnd.x < 0)
                        selectionEnd.x = 0
                    if (selectionEnd.y < 0)
                        selectionEnd.y = 0
                    if (selectionEnd.x > mainScope.width)
                        selectionEnd.x = mainScope.width - selectionArea.touchOffsetX
                    if (selectionEnd.y > mainScope.height)
                        selectionEnd.y = mainScope.height - selectionEnd.height
                }
                selectionArea.updateSelection()
            }
        }
    }

    Row {
        id: selectionButtons
        anchors.horizontalCenter: selectionArea.horizontalCenter
        anchors.top: (selectionArea.x + selectionArea.height + anchors.topMargin < mainScope.height) ? selectionArea.bottom : (selectionArea.y < 0 ? selectionArea.bottom : selectionArea.top)
        anchors.topMargin: 50
        visible: selectionStart.visible && !textinputButtons.visible
        height: copyText.height
        width: copyText.width + selectHide.width

        EditBubbleButton {
            id: copyText
            text: qsTrId("meefox_common_copy")
            platformStyle: EditBubbleButtonStyle {
                position: "horizontal-left"
            }
            onClicked: {
                selectionArea.copySelection()
            }
        }

        EditBubbleButton {
            id: selectHide
            text: qsTrId("meefox_common_cancel")
            platformStyle: EditBubbleButtonStyle {
                position: "horizontal-right"
            }
            onClicked: selectionArea.hideSelection()
        }
    }

    Row {
        id: textinputButtons
        visible: test.visible
        height: copySelected.height
        property bool forceVisible: false
        property bool canPaste: false
        property bool canCopy: false
        property bool canCut: false

        EditBubbleButton {
            id: copySelected
            text: qsTrId("meefox_common_copy")
            visible: textinputButtons.canCopy || selectionStart.visible
            platformStyle: EditBubbleButtonStyle {
                position: "horizontal-left"
            }
            onClicked: {
                if (selectionStart.visible) {
                    selectionArea.copySelection()
                }
                else {
                    webViewport.child.sendAsyncMessage("embedui:editorcopy", {})
                }
            }
        }

        EditBubbleButton {
            id: pasteSelected
            text: qsTrId("meefox_common_paste")
            visible: textinputButtons.canPaste && QmlHelperTools.clipboard.length > 0
            platformStyle: EditBubbleButtonStyle {
                position: copySelected.visible ? "horizontal-center" : "horizontal-left"
            }
            onClicked: {
                webViewport.child.sendAsyncMessage("embedui:editorpaste", {})
            }
        }

        EditBubbleButton {
            id: selectAll
            text: qsTrId("meefox_common_select_all")
            platformStyle: EditBubbleButtonStyle {
                position: copySelected.visible ? "horizontal-center" : ( pasteSelected.visible ? "horizontal-center" : "horizontal-left" )
            }
            onClicked: {
                webViewport.child.sendAsyncMessage("embedui:editorselectall", {})
            }
        }

        EditBubbleButton {
            id: selectManual
            text: qsTrId("meefox_common_selection")
            platformStyle: EditBubbleButtonStyle {
                position: "horizontal-right"
            }
            onClicked: {
                startSelection(test.x, test.y + 10)
                selectionStart.x = test.x - selectionArea.touchOffsetX
                selectionStart.y = test.y + 10
                selectionEnd.x = test.x + 40
                selectionEnd.y = test.y + 10
                selectionStart.visible = true
                selectionEnd.visible = true
            }
        }
    }

    BusyIndicator {
        anchors.centerIn: parent
        visible: firstLoad
        running: visible
        platformStyle: BusyIndicatorStyle { size: "large" }
    }

    Rectangle {
        id: loadProgress
        anchors.top: parent.top
        anchors.left: parent.left
        height: 5
        width: parent.width / 100 * webViewport.child.loadProgress
        color: "cyan"
        opacity: 0.6
        visible: webViewport.child.loading && !addressBar.visible
        smooth: true
    }

    AddressBar {
        id: addressBar
        anchors.top: parent.top
        width: parent.width
        visible: addressBar.showRecent || addressBar.inputFocus || longTimer.longPressed || toolsMenu.forceVisible
        onInputFocusChanged: {
            if (addressBar.inputFocus) {
                toolsMenu.forceVisible = false
                addressBar.selectAll()
            }
        }
    }

    SearchArea {
        id: searchArea
        anchors.bottom: parent.bottom
        width: parent.width
        visible: false
    }

    ToolsMenu {
        id: toolsMenu
        anchors.bottom: parent.bottom
        anchors.bottomMargin: - toolsMenu.height / 3 * 2
        width: (screen.currentOrientation == Screen.Portrait) ? parent.width : toolsMenu.toolsWidth
        anchors.left: parent.left
        anchors.leftMargin: (screen.currentOrientation == Screen.Portrait) ? 0 : ((parent.width - toolsMenu.toolsWidth) / 2)
        visible: (longTimer.longPressed && !webViewport.textInputElement) || toolsMenu.forceVisible
    }

    FoxMenu {
        id: contextMenu
        onStartSelectionRequested: {
            if (contextMenu.lastContextInfo) {
                startSelection(contextMenu.lastContextInfo.xPos, contextMenu.lastContextInfo.yPos)
                selectionStart.x = contextMenu.lastContextInfo.xPos - 20 - selectionArea.touchOffsetX
                selectionStart.y = contextMenu.lastContextInfo.yPos
                selectionEnd.x = contextMenu.lastContextInfo.xPos + 20 + selectionArea.touchOffsetX
                selectionEnd.y = contextMenu.lastContextInfo.yPos
                selectionStart.visible = true
                selectionEnd.visible = true
            }
        }

        onStartFindOnPageRequested: {
            searchArea.visible = true
            searchArea.doFocus()
        }
    }

    AuthDialog {
        id: authDialog
        acceptButtonText: qsTrId("meefox_common_ok")
        rejectButtonText: qsTrId("meefox_common_cancel")
        property int winid: 0
        function handle(isAccepted) {
            webViewport.child.sendAsyncMessage("authresponse", {
                                                     winid: authDialog.winid,
                                                     checkval: 0,
                                                     accepted: isAccepted,
                                                     username: authDialog.username,
                                                     password: authDialog.password
                                                 })
        }
        onAccepted: handle(true)
        onRejected: handle(false)
    }

    PromptDialog {
        id: promptDialog
        acceptButtonText: qsTrId("meefox_common_ok")
        rejectButtonText: qsTrId("meefox_common_cancel")
        property int winid: 0
        function handle(isAccepted) {
            webViewport.child.sendAsyncMessage("promptresponse", {
                                                     winid: promptDialog.winid,
                                                     checkval: 0,
                                                     accepted: isAccepted,
                                                     promptvalue: promptDialog.prompt
                                                 })            
        }
        onAccepted: handle(true)
        onRejected: handle(false)
    }

    QueryDialog {
        id: confirmDialog
        acceptButtonText: qsTrId("meefox_common_ok")
        rejectButtonText: qsTrId("meefox_common_cancel")
        property int winid: 0
        function handle(isAccepted) {
            webViewport.child.sendAsyncMessage("confirmresponse", {
                                                     winid: confirmDialog.winid,
                                                     checkval: false,
                                                     accepted: isAccepted
                                                 })
        }
        onAccepted: handle(true)
        onRejected: handle(false)
    }

    QueryDialog {
        id: alertDialog
        acceptButtonText: qsTrId("meefox_common_ok")
        rejectButtonText: ""
        property int winid: 0
        function handle() {
            webViewport.child.sendAsyncMessage("alertresponse", {
                                                     winid: alertDialog.winid,
                                                     checkval: false,
                                                     accepted: true
                                                 })}
        onAccepted: handle()
        onRejected: handle()
    }

    PermissionsDialog {
        id: permissionsDialog
        acceptButtonText: qsTrId("meefox_common_accept")
        rejectButtonText: qsTrId("meefox_common_deny")
        property variant uid: 0
        function handle(isAccepted) {
            webViewport.child.sendAsyncMessage("embedui:premissions", {
                                                    allow: isAccepted,
                                                    checkedDontAsk: permissionsDialog.dontAsk,
                                                    id: permissionsDialog.uid
                                                 })
        }
        onAccepted: handle(true)
        onRejected: handle(false)
    }

    QueryDialog {
        id: loginManager
        acceptButtonText: qsTrId("meefox_common_save")
        rejectButtonText: qsTrId("meefox_common_cancel")
        property variant uid: 0
        function handle(isAccepted) {
            webViewport.child.sendAsyncMessage("embedui:login", {
                                                    buttonidx: isAccepted ? 0 : 1,
                                                    id: loginManager.uid
                                                 })
        }
        onAccepted: handle(true)
        onRejected: handle(false)
    }

    FilePicker {
        id: filepicker
        acceptButtonText: qsTrId("meefox_common_ok")
        rejectButtonText: qsTrId("meefox_common_cancel")
        property int winid: 0
        property bool userRequest: false
        property string remoteUrl: ""
        function handle(isAccepted) {
            var pickedItems = []
            if (isAccepted) {
                switch (filepicker.mode) {
                    case 1:
                        pickedItems.push(filepicker.path + "/" + filepicker.filename )
                        break;
                    case 2:
                        pickedItems.push(filepicker.path)
                        break;
                    case 3:
                        pickedItems = root.selectedItems
                        break;
                    default:
                        break
                }
            }
            else {
                pickedItems.push("null")
            }
            if (userRequest) {
                if (isAccepted) {
                    MozContext.sendObserve("embedui:download", { msg: "addDownload", from: filepicker.remoteUrl, to: pickedItems[0] })
                }
            }
            else {
                webViewport.child.sendAsyncMessage("filepickerresponse", {
                                                             winid: filepicker.winid,
                                                             accepted: isAccepted,
                                                             items: pickedItems
                                                         })
            }
        }
        onAccepted: handle(true)
        onRejected: handle(false)
    }

    ComboSelection {
        id: comboSelection
        property bool syncLock: false
        function handle() {
            comboSelection.syncLock = false
        }
        onAccepted: handle()
        onRejected: handle()
    }

    InfoBanner {
        id: infoBanner
    }
}
