import QtQuick 1.1
import com.nokia.meego 1.0

Rectangle {
    id: root
    color: theme.inverted ? "black" : "white"
    height: searchText.height + 20

    function doSearch(text, isAgain, isBackwards) {
        webViewport.child.sendAsyncMessage("embedui:find", { text: text, backwards: isBackwards, again: isAgain })
    }

    function showBanner(text, icon) {
        infoBanner.text = text
        infoBanner.iconSource = icon
        infoBanner.show()
    }

    function doFocus() {
        searchText.forceActiveFocus()
    }

    Connections {
        target: webViewport.child
        onRecvAsyncMessage: {
            switch (message) {
                case "embed:find": {
                    switch (data.r) {
                      case 0: {
                        //console.log("Page Find: found");
                        break;
                      }
                      case 1: {
                        //console.log("Page Find:  not found")
                        showBanner(qsTrId("meefox_search_notfound"), mainPage.iconset + "/context-search.png")
                        break;
                      }
                      case 2: {
                        //console.log("Page Find: found, wrapped");
                        showBanner(qsTrId("meefox_search_wrapped"), mainPage.iconset + "/context-search.png")
                        break;
                      }
                      case 3: {
                        //console.log("Page Find: pending");
                        break;
                      }
                    }
                    break;
                }
            }
        }
    }

    Rectangle {
        id: separator
        anchors.top: parent.top
        width: parent.width
        color: theme.inverted ? "white" : "black"
    }

    FoxTextField {
        id: searchText
        anchors.verticalCenter: root.verticalCenter
        anchors.left: parent.left
        anchors.leftMargin: 10
        anchors.right: doSearchAgain.left
        anchors.rightMargin: 10

        Keys.onReturnPressed: {
            root.doSearch(searchText.text, false, false)
        }
    }

    Button {
        id: doSearchAgain
        height: searchText.height
        width: height
        anchors.verticalCenter: root.verticalCenter
        anchors.right: doSearchBackwards.left
        anchors.rightMargin: 10
        iconSource: mainPage.iconset + "/search-next.png"

        onClicked: {
            root.doSearch(searchText.text, true, false)
        }
    }

    Button {
        id: doSearchBackwards
        height: searchText.height
        width: height
        anchors.verticalCenter: root.verticalCenter
        anchors.right: closeSearch.left
        anchors.rightMargin: 10
        iconSource: mainPage.iconset + "/search-back.png"

        onClicked: {
            root.doSearch(searchText.text, true, true)
        }
    }

    Button {
        id: closeSearch
        height: searchText.height
        width: height
        anchors.verticalCenter: root.verticalCenter
        anchors.right: parent.right
        anchors.rightMargin: 10
        iconSource: mainPage.iconset + "/close.png"

        onClicked: {
            root.doSearch("", false, false)
            root.visible = false
        }
    }
} 
