import QtQuick 1.1
import com.nokia.meego 1.0

Page {
    id: root
    tools: ToolBarLayout {
        ToolIcon { iconId: "toolbar-back"; onClicked: {
                pageStack.pop()
                MozContext.sendObserve("embedui:saveprefs", {})
            }
        }
        ButtonRow {
            style: TabButtonStyle { }
            TabButton {
                iconSource: "image://theme/icon-m-toolbar-settings" + (theme.inverted ? "-white" : "")
                tab: tab1
                onClicked: {
                    MozContext.sendObserve("embedui:prefs", { msg: "getPrefList", prefs: [ "general.useragent.override",
                                                                                           "browser.zoom.reflowOnZoom",
                                                                                           "browser.zoom.reflowMobilePages"]})
                }
            }
            TabButton {
                iconSource: "image://theme/icon-m-toolbar-tools" + (theme.inverted ? "-white" : "")
                tab: tab2
                onClicked: {
                    MozContext.sendObserve("embedui:prefs", { msg: "getPrefList", prefs: [ "browser.ui.touch.top",
                                                                                           "browser.ui.touch.left",
                                                                                           "browser.ui.touch.bottom",
                                                                                           "browser.ui.touch.right",
                                                                                           "gfx.azpc.vertical_scroll_lock_ratio",
                                                                                           "gfx.azpc.horizontal_scroll_lock_ratio"]})
                }
            }
            TabButton {
                iconSource: "image://theme/icon-m-toolbar-frequent-used" + (theme.inverted ? "-white" : "")
                tab: tab3
            }
         }
        ToolIcon { iconId: "toolbar-list"; onClicked: {
                MozContext.sendObserve("embedui:allprefs", {})
                pageStack.push(configPage)
            }
        }
    }

    function saveSetting(name, value) {
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                tx.executeSql('delete from settings where name=(?);',[name])
                tx.executeSql('INSERT INTO settings VALUES (?,?);',[name,value])
            }
        );
    }

    function addIconset(set) {
        tab1.addIconset(set)
    }

    Component.onCompleted: {
        print("Settings ready!")
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                tx.executeSql('CREATE TABLE IF NOT EXISTS settings (name TEXT, value TEXT)')
            }
        );
        var items = null
        db.transaction(
            function(tx) {
                var result = tx.executeSql('select * from settings')
                items = result.rows
            }
        );
        if (items.length < 1) {
            console.log("Settings empty!")
            db.transaction(
                function(tx) {
                    tx.executeSql('INSERT INTO settings VALUES (?,?);',["theme_inverted","1"])
                    tx.executeSql('INSERT INTO settings VALUES (?,?);',["touch_offset_x","0"])
                    tx.executeSql('INSERT INTO settings VALUES (?,?);',["touch_offset_y","0"])
                    tx.executeSql('INSERT INTO settings VALUES (?,?);',["longtap_delay","800"])
                    tx.executeSql('INSERT INTO settings VALUES (?,?);',["longtap_distance","15"])
                    tx.executeSql('INSERT INTO settings VALUES (?,?);',["theme_iconset","/opt/meefox/icons/inverted"])
                    tx.executeSql('INSERT INTO settings VALUES (?,?);',["status_bar","0"])
                }
            );
        }
        else {
            for (var i=0; i < items.length; i++) {
                var item = items.item(i)
                switch (item.name) {
                    case "status_bar": {
                        appWindow.showStatusBar = item.value == "1"
                        break
                    }
                    case "theme_inverted": {
                        theme.inverted = item.value == "1"
                        break
                    }
                    case "touch_offset_x": {
                        mainPage.touchPointOffsetX = parseInt(item.value)
                        break
                    }
                    case "touch_offset_y": {
                        mainPage.touchPointOffsetY = parseInt(item.value)
                        break
                    }
                    case "longtap_delay": {
                        mainPage.longTimerInterval = parseInt(item.value)
                        break
                    }
                    case "longtap_distance": {
                        mainPage.longtapCancelDistance = parseInt(item.value)
                        break
                    }
                    case "theme_iconset": {
                        mainPage.iconset = item.value
                        break
                    }
                }
            }
        }
    }

    function show() {
        MozContext.sendObserve("embedui:prefs", { msg: "getPrefList", prefs: [ "general.useragent.override",
                                                                               "browser.zoom.reflowOnZoom",
                                                                               "browser.zoom.reflowMobilePages"]})
        MozContext.sendObserve("embedui:search", { msg: "getlist"})
    }

    TabGroup {
        id: tabGroup

        currentTab: tab1

        SettingsTabMain {
            id: tab1
        }
        SettingsTabOther {
            id: tab2
        }
        SettingsTabAbout {
            id: tab3
        }
    }

    Connections {
        target: tab1
        onIconsetChanged: {
            mainPage.iconset = iconSet
        }
    }

    Connections {
        target: mainPage
        onIconsetNameChanged: {
            tab1.setIconsetName(mainPage.iconsetName)
        }
    }
} 
