import QtQuick 1.1
import com.nokia.meego 1.0

Page {
    id: root
    signal iconsetChanged(string iconSet)

    function addIconset(set) {
        iconsetModel.append(set)
    }

    Connections {
        target: MozContext
        onRecvObserve: {
            switch (message) {
                case "embed:prefs": {
                    for (var i=0; i<data.length; i++) {
                        switch (data[i].name) {
                            case "general.useragent.override": {
                                uaString.text = data[i].value;
                                uaString.cursorPosition = 0;
                                customUA.checked = uaString.text.length > 0
                                predefinedUA.text = checkUA(data[i].value)
                                break;
                            }
                            case "browser.zoom.reflowOnZoom": {
                                zoomReflow.checked = data[i].value;
                                break;
                            }
                            case "browser.zoom.reflowMobilePages": {
                                mobileReflow.checked = data[i].value;
                                break;
                            }
                        }
                    }
                    break
                }
                case "embed:search": {
                    switch (data.msg) {
                        case "init": {
                            if (data.defaultEngine == null) {
                                MozContext.sendObserve("embedui:search", {msg:"setcurrent", name:"Google"})
                                MozContext.sendObserve("embedui:search", {msg:"setdefault", name:"Google"})
                                selectSearchEngine.text = "Google"
                            }
                            else {
                                selectSearchEngine.text = data.defaultEngine
                            }
                            break
                        }
                        case "pluginslist":
                        {
                            searchEnginesModel.clear()
                            for (var i=0; i<data.list.length; i++) {
                                var plugin = data.list[i]
                                if (plugin.isDefault) {
                                    selectSearchEngine.text = plugin.name
                                }
                                searchEnginesModel.append({name: plugin.name})
                            }
                            break
                        }
                    }
                    break
                }
            }
        }
    }

    function setIconsetName(iconset) {
        selectIconset.text = iconset
    }

    PageHeader {
        id: header
        anchors.top: root.top
        width: root.width
        titleText: qsTrId("meefox_settings_main")
        iconSource: mainPage.iconset + "/settings.png"
    }

    Flickable {
        id: container
        clip: true
        anchors.top: header.bottom
        anchors.left: root.left
        anchors.leftMargin: 10
        anchors.right: root.right
        anchors.rightMargin: 10
        anchors.bottom: root.bottom
        contentHeight: content.height

        Column {
            id: content
            width: parent.width
            spacing: 5

            Text {
                text: qsTrId("meefox_settings_ua_custom")
                font.pixelSize: 26
                color: theme.inverted ? "white" : "black"
            }

            Item {
                id: uaItem
                width: parent.width
                height: Math.max(customUA.height, uaString.height) + 10

                FoxSwitch {
                    id: customUA
                    anchors.left: parent.left
                    anchors.verticalCenter: parent.verticalCenter
                    onChanged: {
                        if (checked && uaString.text.length > 0) {
                            MozContext.setPref("general.useragent.override", uaString.text)
                            MozContext.sendObserve("embedui:useragent", {useragent: uaString.text})
                            predefinedUA.text = checkUA(uaString.text)
                        }
                        else {
                            MozContext.sendObserve("embedui:clearprefs", { prefs : ["general.useragent.override"] })
                            MozContext.sendObserve("embedui:useragent", {useragent: null})
                        }
                    }
                }

                FoxTextField {
                    id: uaString
                    anchors.left: customUA.right
                    anchors.leftMargin: 15
                    anchors.right: parent.right
                    anchors.rightMargin: 10
                    anchors.verticalCenter: parent.verticalCenter
                    text: "Mozilla/5.0 (X11; Linux x86_64; rv:20.0) Gecko/20130124 Firefox/20.0"
                    Keys.onReturnPressed: {
                        if (uaString.text.length > 0) {
                            MozContext.setPref("general.useragent.override", uaString.text)
                            MozContext.sendObserve("embedui:useragent", {useragent: uaString.text})
                            customUA.checked = true
                            predefinedUA.text = qsTrId("meefox_settings_ua_custom")
                            uaString.cursorPosition = 0
                        }
                        else {
                            MozContext.sendObserve("embedui:clearprefs", { prefs : ["general.useragent.override"] })
                            MozContext.sendObserve("embedui:useragent", {useragent: null})
                            customUA.checked = false
                        }
                    }
                }
            }

            Button {
                id: predefinedUA
                width: parent.width
                text: qsTrId("meefox_settings_ua_custom")
                onClicked: {
                    uaDialog.open()
                }
            }

            LabeledSwitch {
                id: zoomReflow
                width: parent.width
                text: qsTrId("meefox_settings_reflow_text")
                color: theme.inverted ? "white" : "black"
                onChanged: {
                    MozContext.setPref("browser.zoom.reflowOnZoom", checked)
                }
            }

            LabeledSwitch {
                id: mobileReflow
                width: parent.width
                text: qsTrId("meefox_settings_reflow_mobile")
                color: theme.inverted ? "white" : "black"
                onChanged: {
                    MozContext.setPref("browser.zoom.reflowMobilePages", checked)
                }
            }

            Text {
                id: searchEnginesText
                text: qsTrId("meefox_settings_search_engine")
                font.pixelSize: 26
                color: theme.inverted ? "white" : "black"
            }

            Button {
                id: selectSearchEngine
                width: parent.width
                function callback() {
                    selectSearchEngine.text = selectDialog.selectedName
                    selectDialog.accepted.disconnect(selectSearchEngine.callback)
                    MozContext.sendObserve("embedui:search", {msg:"setcurrent", name:text})
                    MozContext.sendObserve("embedui:search", {msg:"setdefault", name:text})
                }
                onClicked: {
                    selectDialog.model = searchEnginesModel
                    selectDialog.open()
                    selectDialog.accepted.connect(selectSearchEngine.callback)
                }
            }

            /*Button {
                width: parent.width
                text: "Test Sync"
                onClicked: {
                    MozContext.sendObserve("embedui:initsync", { username: "weave username hash", password: "password", key: "recovery key" })
                }
            }*/

            LabeledSwitch {
                id: themeInverted
                width: parent.width
                text: (theme.inverted ? qsTrId("meefox_settings_theme_inverted") : qsTrId("meefox_settings_theme_normal"))
                color: theme.inverted ? "white" : "black"
                checked: theme.inverted
                onChanged: {
                    theme.inverted = themeInverted.checked
                    settingsPage.saveSetting("theme_inverted", themeInverted.checked ? "1" : "0")
                }
            }

            LabeledSwitch {
                id: showStatusArea
                width: parent.width
                text: qsTrId("meefox_settings_status_show")
                color: theme.inverted ? "white" : "black"
                checked: appWindow.showStatusBar
                onChanged: {
                    appWindow.showStatusBar = showStatusArea.checked
                    settingsPage.saveSetting("status_bar", showStatusArea.checked ? "1" : "0")
                }
            }

            Text {
                id: selectIconsetText
                text: qsTrId("meefox_settings_icons_select")
                font.pixelSize: 26
                color: theme.inverted ? "white" : "black"
            }

            Button {
                id: selectIconset
                width: parent.width
                text: "inverted"
                function callback() {
                    selectIconset.text = selectDialog.selectedTitle
                    selectDialog.accepted.disconnect(selectIconset.callback)
                    root.iconsetChanged(selectDialog.selectedName)
                    settingsPage.saveSetting("theme_iconset", selectDialog.selectedName)
                }
                onClicked: {
                    selectDialog.model = iconsetModel
                    selectDialog.titleText = qsTrId("meefox_dialog_iconset")
                    selectDialog.open()
                    selectDialog.accepted.connect(selectIconset.callback)
                }
            }
        }
    }

    ScrollDecorator {
        flickableItem: container
    }

    ListModel {
        id: searchEnginesModel
    }

    ListModel {
        id: iconsetModel
    }

    function checkUA(text) {
        for (var i=0; i<uaModel.count; i++) {
            var model = uaModel.get(i)
            if (model.name == text) {
                return model.title;
            }
        }
        return qsTrId("meefox_settings_ua_custom");
    }

    ListModel {
        id: uaModel
        ListElement {
            title: "iOS 6"
            name: "Mozilla/5.0 (iPhone; CPU iPhone OS 6_0 like Mac OS X) AppleWebKit/536.26 (KHTML, like Gecko) Version/6.0 Mobile/10A5376e Safari/8536.25"
        }
        ListElement {
            title: "iPhone 4"
            name: "Mozilla/5.0 (iPhone; U; CPU iPhone OS 4_0 like Mac OS X; en-us) AppleWebKit/532.9 (KHTML, like Gecko) Version/4.0.5 Mobile/8A293 Safari/6531.22.7"
        }
        ListElement {
            title: "Android 4.x Chrome (on Galaxy Nexus)"
            name: "Mozilla/5.0 (Linux; Android 4.0.4; Galaxy Nexus Build/IMM76B) AppleWebKit/535.19 (KHTML, like Gecko) Chrome/18.0.1025.133 Mobile Safari/535.19"
        }
        ListElement {
            title: "Android 4.x Firefox"
            name: "Mozilla/5.0 (Android; Mobile; rv:13.0) Gecko/13.0 Firefox/13.0"
        }
        ListElement {
            title: "Linux Firefox Desktop"
            name: "Mozilla/5.0 (X11; Linux i686; rv:10.0) Gecko/20100101 Firefox/10.0"
        }
        ListElement {
            title: "N9 Grob"
            name: "Mozilla/5.0 (MeeGo; NokiaN9) AppleWebKit/534.13 (KHTML, like Gecko) NokiaBrowser/8.5.0 Mobile Safari/534.13"
        }
        ListElement {
            title: "Blackberry Z10"
            name: "Mozilla/5.0 (BB10; Touch) AppleWebKit/537.10 (KHTML, like Gecko) Version/10.0.9.930 Mobile Safari/537.10"
        }
        ListElement {
            title: "Firefox OS"
            name: "Mozilla/5.0 (Mobile; rv:15.0) Gecko/15.0 Firefox/15.0"
        }
        ListElement {
            title: "Nokia N900 MicroB"
            name: "Mozilla/5.0 (Maemo; Linux armv7l; rv:10.0) Gecko/20100101 Firefox/10.0 Fennec/10.0"
        }
        ListElement {
            title: "Lumia 800"
            name: "Mozilla/5.0 (compatible; MSIE 9.0; Windows Phone OS 7.5; Trident/5.0; IEMobile/9.0; NOKIA; Lumia 800)"
        }
        ListElement {
            title: "808 PureView"
            name: "Mozilla/5.0 (Symbian/3; Series60/5.5 Nokia808PureView/113.010.1507; Profile/MIDP-2.1 Configuration/CLDC-1.1 ) AppleWebKit/535.1 (KHTML, like Gecko) NokiaBrowser/8.3.2.21 Mobile Safari/535.1"
        }
        ListElement {
            title: "Lumia 925"
            name: "Mozilla/5.0 (compatible; MSIE 10.0; Windows Phone 8.0; Trident/6.0; IEMobile/10.0; ARM; Touch; NOKIA; Lumia 925)"
        }
    }

    ListSelectionDialog {
        id: selectDialog
        canAdd: false
        acceptButtonText: ""
        rejectButtonText: qsTrId("meefox_common_cancel")
    }

    ListSelectionDialog {
        id: uaDialog
        canAdd: true
        titleText: qsTrId("meefox_dialog_useragent")
        acceptButtonText: ""
        rejectButtonText: qsTrId("meefox_common_cancel")
        model: uaModel
        onAccepted: {
            predefinedUA.text = uaDialog.selectedTitle
            uaString.text = uaDialog.selectedName
            uaString.cursorPosition = 0
            MozContext.setPref("general.useragent.override", uaDialog.selectedName)
            MozContext.sendObserve("embedui:useragent", {useragent: uaDialog.selectedName})
            customUA.checked = true
        }
    }
} 
