import QtQuick 1.1
import com.nokia.meego 1.0

Page {
    id: root

    Connections {
        target: MozContext
        onRecvObserve: {
            switch (message) {
                case "embed:prefs": {
                    for (var i=0; i<data.length; i++) {
                        switch (data[i].name) {
                            case "gfx.azpc.vertical_scroll_lock_ratio": {
                                verticalScrollLockRatio.text = data[i].value.replace("f", "");
                                verticalScrollLockRatio.cursorPosition = 0;
                                break;
                            }
                            case "gfx.azpc.horizontal_scroll_lock_ratio": {
                                horizontalScrollLockRatio.text = data[i].value.replace("f", "");
                                horizontalScrollLockRatio.cursorPosition = 0;
                                break;
                            }
                            case "browser.ui.touch.left": {
                                touchLeft.x = (touchRadius.width / 2 ) - parseInt(data[i].value)
                                break;
                            }
                            case "browser.ui.touch.right": {
                                touchRight.x = (touchRadius.width / 2 ) + parseInt(data[i].value)
                                break;
                            }
                            case "browser.ui.touch.top": {
                                touchTop.y = (touchRadius.height / 2 ) - parseInt(data[i].value)
                                break;
                            }
                            case "browser.ui.touch.bottom": {
                                touchBottom.y = (touchRadius.height / 2 ) + parseInt(data[i].value)
                                break;
                            }
                        }
                    }
                    break
                }
            }
        }
    }

    PageHeader {
        id: header
        anchors.top: root.top
        width: root.width
        titleText: qsTrId("meefox_settings_other")
        iconSource: mainPage.iconset + "/settings.png"
    }

    Flickable {
        id: container
        clip: true
        anchors.top: header.bottom
        anchors.left: root.left
        anchors.leftMargin: 10
        anchors.right: root.right
        anchors.rightMargin: 10
        anchors.bottom: root.bottom
        contentHeight: content.height
        interactive: !touchRadiusMouseArea.pressed

        Column {
            id: content
            width: parent.width
            spacing: 5

            Text {
                id: verticalScrollLockText
                text: qsTrId("meefox_settings_vertical_lock")
                font.pixelSize: 26
                color: theme.inverted ? "white" : "black"
            }

            FoxTextField {
                id: verticalScrollLockRatio
                width: parent.width-1
                text: "1.2"
                inputMethodHints: Qt.ImhDigitsOnly
                Keys.onReturnPressed: {
                    MozContext.setPref("gfx.azpc.vertical_scroll_lock_ratio", verticalScrollLockRatio.text + "f")
                }
            }

            Text {
                id: horizontalScrollLockText
                text: qsTrId("meefox_settings_horizontal_lock")
                font.pixelSize: 26
                color: theme.inverted ? "white" : "black"
            }

            FoxTextField {
                id: horizontalScrollLockRatio
                width: parent.width-1
                text: "0.5"
                inputMethodHints: Qt.ImhDigitsOnly
                Keys.onReturnPressed: {
                    MozContext.setPref("gfx.azpc.horizontal_scroll_lock_ratio", horizontalScrollLockRatio.text + "f")
                }
            }

            Text {
                id: longTapCancelDistanceText
                text: qsTrId("meefox_settings_longtap_cancel")
                font.pixelSize: 26
                color: theme.inverted ? "white" : "black"
            }

            FoxTextField {
                id: longTapCancelDistance
                width: parent.width-1
                text: mainPage.longtapCancelDistance
                inputMethodHints: Qt.ImhDigitsOnly
                Keys.onReturnPressed: {
                    mainPage.longtapCancelDistance = parseInt(longTapCancelDistance.text)
                    settingsPage.saveSetting("longtap_distance", longTapCancelDistance.text)
                }
            }

            Text {
                id: longTapDelayText
                text: qsTrId("meefox_settings_longtap_delay")
                font.pixelSize: 26
                color: theme.inverted ? "white" : "black"
            }

            FoxTextField {
                id: longTapDelay
                width: parent.width-1
                text: mainPage.longTimerInterval
                inputMethodHints: Qt.ImhDigitsOnly
                Keys.onReturnPressed: {
                    mainPage.longTimerInterval = parseInt(longTapDelay.text)
                    settingsPage.saveSetting("longtap_delay", longTapDelay.text)
                }
            }

            Text {
                id: touchRadiusText
                text: qsTrId("meefox_settings_touch_radius")
                font.pixelSize: 26
                color: theme.inverted ? "white" : "black"
            }

            Item {
                id: touchRadius
                width: parent.width
                height: 150
                property int touchTapTop: (height / 2) - touchTop.y
                property int touchTapBottom: touchBottom.y - (height / 2)
                property int touchTapLeft: (width / 2) - touchLeft.x
                property int touchTapRight: touchRight.x - (width / 2)

                Rectangle {
                    width: 3
                    height: 3
                    radius: 2
                    smooth: true
                    color: "red"
                    x: touchRadius.width / 2 - 1
                    y: touchRadius.height / 2 - 1
                }

                Rectangle {
                    id: touchTop
                    y: (touchRadius.height / 2) - 1
                    anchors.left: touchLeft.right
                    anchors.right: touchRight.left
                    height: 1
                    color: theme.inverted ? "white" : "black"
                }

                Rectangle {
                    id: touchBottom
                    y: (touchRadius.height / 2) + 1
                    anchors.left: touchLeft.right
                    anchors.right: touchRight.left
                    height: 1
                    color: theme.inverted ? "white" : "black"
                }

                Rectangle {
                    id: touchLeft
                    x: (touchRadius.width / 2) - 1
                    anchors.top: touchTop.bottom
                    anchors.bottom: touchBottom.top
                    width: 1
                    color: theme.inverted ? "white" : "black"
                }

                Rectangle {
                    id: touchRight
                    x: (touchRadius.width / 2) + 1
                    anchors.top: touchTop.bottom
                    anchors.bottom: touchBottom.top
                    width: 1
                    color: theme.inverted ? "white" : "black"
                }

                MouseArea {
                    id: touchRadiusMouseArea
                    anchors.fill: parent
                    function handleMouse(mouse) {
                        if (mouse.y < touchTop.y + 10 && mouse.y > touchTop.y - 10) {
                            touchTop.y = mouse.y
                            MozContext.setPref("browser.ui.touch.top", touchRadius.touchTapTop)
                        }
                        if (mouse.y < touchBottom.y + 10 && mouse.y > touchBottom.y - 10) {
                            touchBottom.y = mouse.y
                            MozContext.setPref("browser.ui.touch.bottom", touchRadius.touchTapBottom)
                        }
                        if (mouse.x < touchLeft.x + 10 && mouse.x > touchLeft.x - 10) {
                            touchLeft.x = mouse.x
                            MozContext.setPref("browser.ui.touch.left", touchRadius.touchTapLeft)
                        }
                        if (mouse.x < touchRight.x + 10 && mouse.x > touchRight.x - 10) {
                            touchRight.x = mouse.x
                            MozContext.setPref("browser.ui.touch.right", touchRadius.touchTapRight)
                        }
                    }
                    onPositionChanged: handleMouse(mouse)
                    onPressed: handleMouse(mouse)
                }

                Text {
                    font.pixelSize: 20
                    color: theme.inverted ? "white" : "black"
                    anchors.top: parent.top
                    anchors.horizontalCenter: parent.horizontalCenter
                    text: touchRadius.touchTapTop
                    MouseArea {
                        anchors.fill: parent
                        onDoubleClicked: {
                            promptDialog.prompt = parent.text
                            promptDialog.titleText = qsTrId("meefox_settings_radius_top")
                            promptDialog.index = 0
                            promptDialog.open()
                        }
                    }
                }

                Text {
                    font.pixelSize: 20
                    color: theme.inverted ? "white" : "black"
                    anchors.bottom: parent.bottom
                    anchors.horizontalCenter: parent.horizontalCenter
                    text: touchRadius.touchTapBottom
                    MouseArea {
                        anchors.fill: parent
                        onDoubleClicked: {
                            promptDialog.prompt = parent.text
                            promptDialog.titleText = qsTrId("meefox_settings_radius_bottom")
                            promptDialog.index = 1
                            promptDialog.open()
                        }
                    }
                }

                Text {
                    font.pixelSize: 20
                    color: theme.inverted ? "white" : "black"
                    anchors.left: parent.left
                    anchors.verticalCenter: parent.verticalCenter
                    text: touchRadius.touchTapLeft
                    MouseArea {
                        anchors.fill: parent
                        onDoubleClicked: {
                            promptDialog.prompt = parent.text
                            promptDialog.titleText = qsTrId("meefox_settings_radius_left")
                            promptDialog.index = 2
                            promptDialog.open()
                        }
                    }
                }

                Text {
                    font.pixelSize: 20
                    color: theme.inverted ? "white" : "black"
                    anchors.right: parent.right
                    anchors.verticalCenter: parent.verticalCenter
                    text: touchRadius.touchTapRight
                    MouseArea {
                        anchors.fill: parent
                        onDoubleClicked: {
                            promptDialog.prompt = parent.text
                            promptDialog.titleText = qsTrId("meefox_settings_radius_right")
                            promptDialog.index = 3
                            promptDialog.open()
                        }
                    }
                }
            }

            Text {
                id: touchOffsetText
                text: qsTrId("meefox_settings_tuning")
                font.pixelSize: 26
                color: theme.inverted ? "white" : "black"
            }

            AxisTestItem {
                id: axisOffsetDisplay
                width: parent.width
                height: 150
                simple: true
                active: false
                newAnchorY: mainPage.touchPointOffsetY
                MouseArea {
                    anchors.fill: parent
                    onClicked: {
                        if (axisOffsetDisplay.simple) {
                            settingsPage.pageStack.push(touchTest)
                        }
                        else {
                            settingsPage.pageStack.push(dynamicTouchTest)
                        }
                    }
                }
            }

            ButtonRow {
                width: parent.width

                Button {
                    text: qsTrId("meefox_settings_tuning_simple")
                    onClicked: axisOffsetDisplay.simple = true
                }

                Button {
                    text: qsTrId("meefox_settings_tuning_complex")
                    onClicked: axisOffsetDisplay.simple = false
                }
            }
        }
    }

    ScrollDecorator {
        flickableItem: container
    }

    PromptDialog {
        id: promptDialog
        titleText: ""
        message: qsTrId("meefox_dialog_radius")
        acceptButtonText: qsTrId("meefox_common_ok")
        rejectButtonText: qsTrId("meefox_common_cancel")
        inputMethodHints: Qt.ImhDigitsOnly
        prompt: ""
        property int index: -1
        onAccepted: {
            switch (promptDialog.index) {
                case 0: {
                    touchTop.y = (touchRadius.height / 2 ) - parseInt(promptDialog.prompt)
                    MozContext.setPref("browser.ui.touch.top", touchRadius.touchTapTop)
                    break
                }
                case 1: {
                    touchBottom.y = (touchRadius.height / 2 ) + parseInt(promptDialog.prompt)
                    MozContext.setPref("browser.ui.touch.bottom", touchRadius.touchTapBottom)
                    break
                }
                case 2: {
                    touchLeft.x = (touchRadius.width / 2 ) - parseInt(promptDialog.prompt)
                    MozContext.setPref("browser.ui.touch.left", touchRadius.touchTapLeft)
                    break
                }
                case 3: {
                    touchRight.x = (touchRadius.width / 2 ) + parseInt(promptDialog.prompt)
                    MozContext.setPref("browser.ui.touch.right", touchRadius.touchTapRight)
                    break
                }
            }
        }
    }
} 
