import QtQuick 1.1
import com.nokia.meego 1.0

Page {
    id: root
    property bool showRecent: false
    property bool firstLoad: true

    /*PageHeader {
        id: header
        anchors.top: root.top
        width: root.width
        titleText: "Start"
        iconSource: mainPage.iconset + "/home.png"
    }*/

    Timer {
        id: selectAllTimer
        interval: 500
        repeat: false
        onTriggered: {
            address.cursorPosition = address.text.length
            address.platformOpenSoftwareInputPanel()
            address.selectAll()
        }
    }

    function getRecentCount(count) {
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                var result = tx.executeSql('select * from history order by date desc limit (?)', count)
                recentListModel.clear()
                for (var i=result.rows.length; i > 0; i--) {
                    recentListModel.insert(0, {"url": result.rows.item(i-1).url,
                                     "title": result.rows.item(i-1).title,
                                     "icon": result.rows.item(i-1).icon})
                }
            }
        );
    }

    function getStartPageBookmarks() {
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                var result = tx.executeSql('SELECT * FROM bookmarks where type=1 order by category desc')
                bookmarksListModel.clear()
                for (var i=0; i < result.rows.length; i++) {
                    bookmarksListModel.insert(0, {"url": result.rows.item(i).url,
                                                  "title": result.rows.item(i).title,
                                                  "icon": result.rows.item(i).icon,
                                                  "group": result.rows.item(i).category,
                                                  "type": parseInt(result.rows.item(i).type)})
                }
            }
        );
    }

    function show() {
        getRecentCount(10)
        getStartPageBookmarks()
    }

    function hideRecentList() {
        root.showRecent = false
        recentSitesList.height = 0
    }

    function fillRecentFromDatabase(value) {
        var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
        db.transaction(
            function(tx) {
                var result = tx.executeSql('SELECT url, title, icon FROM history where url like (?) order by date desc limit 5',["%" + value + "%"])
                addressListModel.clear()
                for (var i=0; i < result.rows.length; i++) {
                    addressListModel.insert(0, {"url": result.rows.item(i).url,
                                     "title": result.rows.item(i).title,
                                     "icon": result.rows.item(i).icon})
                }
            }
        );
    }

    Connections {
        target: mainPage.viewportChild

        onUrlChanged: {
            if (firstLoad)
                firstLoad = false
            address.text = mainPage.viewportChild.url;
            address.cursorPosition = 0;
        }
    }

    Connections {
        target: bookmarksPage

        onBookmarksChanged: {
            getStartPageBookmarks()
        }
    }

    ListModel {
        id: bookmarksListModel
    }

    ListModel {
        id: recentListModel
    }

    FoxTextField {
        id: address
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        anchors.margins: 5
        inputMethodHints: Qt.ImhNoPredictiveText || Qt.ImhNoAutoUppercase
        onActiveFocusChanged: {
            if (address.activeFocus) {
                selectAllTimer.start()
            }
        }
        Keys.onReturnPressed: {
            mainPage.load(text)
            pageStack.pop()
        }
        onTextChanged: {
            if (address.activeFocus) {
                fillRecentFromDatabase(text)
                root.showRecent = (addressListModel.count > 0)
                if (addressListModel.count > 0) {
                    var url0 = addressListModel.get(0).url
                    if (url0.search(text) == 0) {
                        address.setUrl(url0)
                    }
                }
                if (addressListModel.count > 4) {
                    recentSitesList.height = 200
                }
                else {
                    recentSitesList.height = (50 * addressListModel.count)
                }
            }
        }
    }

    Text {
        id: bookmarksTitle
        text: qsTrId("meefox_start_pinned")
        font.pixelSize: 26
        anchors.top: address.bottom
        anchors.topMargin: 5
        anchors.left: root.left
        anchors.right: root.right
        anchors.rightMargin: (screen.currentOrientation == Screen.Portrait) ? 0 : (parent.width / 2)
        color: theme.inverted ? "white" : "black"
    }

    ListView {
        id: bookmarksList
        anchors.top: bookmarksTitle.bottom
        anchors.left: root.left
        anchors.right: root.right
        anchors.rightMargin: (screen.currentOrientation == Screen.Portrait) ? 0 : (parent.width / 2)
        height: (screen.currentOrientation == Screen.Portrait) ?
                ((parent.height - address.height - bookmarksTitle.height - recentTitle.height - 20 - toolsMenu.height) / 2) :
                (parent.height - address.height - bookmarksTitle.height - toolsMenu.height - 18)
        clip: true
        model: bookmarksListModel
        delegate: Item {
            width: parent.width
            height: 60
            Rectangle {
                anchors.fill: parent
                color: mArea.pressed ? ( theme.inverted ? "#606060" : "#a0a0a0" ) : ( theme.inverted ? ( index%2==1 ? "#202020" : "transparent" ) : ( index%2==1 ? "#d0d0d0" : "transparent" ) )
            }
            Image {
                id: siteIcon
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: parent.left
                anchors.leftMargin: 3
                source: model.icon ? (testIcon.status == Image.Error ? QmlHelperTools.getFaviconFromUrl(model.url) : model.icon) : QmlHelperTools.getFaviconFromUrl(model.url)
                width: 20
                height: 20
                smooth: true
                cache: true
            }
            Image {
                id: testIcon
                visible: false
                source: model.icon
            }
            Text {
                id: siteLabel
                anchors.top: parent.top
                anchors.topMargin: 3
                anchors.left: siteIcon.right
                anchors.leftMargin: 10
                text: model.title
                font.pixelSize: 18
                color: theme.inverted ? "white" : "black"
            }
            Text {
                id: siteUrl
                anchors.top: siteLabel.bottom
                anchors.topMargin: 1
                anchors.left: siteIcon.right
                anchors.leftMargin: 10
                text: model.url
                font.underline: true
                font.pixelSize: 18
                color: theme.inverted ? "white" : "black"
            }
            MouseArea {
                id: mArea
                anchors.fill: parent
                onClicked: {
                    mainPage.load(model.url)
                    pageStack.pop()
                }
            }
        }
    }

    ScrollDecorator {
        flickableItem: bookmarksList
    }

    Text {
        id: recentTitle
        text: qsTrId("meefox_start_recent")
        font.pixelSize: 26
        anchors.top: (screen.currentOrientation == Screen.Portrait) ? bookmarksList.bottom : address.bottom
        anchors.topMargin: 5
        anchors.left: root.left
        anchors.leftMargin: (screen.currentOrientation == Screen.Portrait) ? 0 : (parent.width / 2)
        anchors.right: root.right
        color: theme.inverted ? "white" : "black"
    }

    ListView {
        id: recentList
        anchors.top: recentTitle.bottom
        anchors.left: root.left
        anchors.leftMargin: (screen.currentOrientation == Screen.Portrait) ? 0 : (parent.width / 2)
        anchors.right: root.right
        height: (screen.currentOrientation == Screen.Portrait) ?
                ((parent.height - address.height - bookmarksTitle.height - recentTitle.height - 20 - toolsMenu.height) / 2) :
                (parent.height - address.height - bookmarksTitle.height - toolsMenu.height - 18)
        clip: true
        model: recentListModel
        delegate: Item {
            width: parent.width
            height: 60
            Rectangle {
                anchors.fill: parent
                color: mArea.pressed ? ( theme.inverted ? "#606060" : "#a0a0a0" ) : ( theme.inverted ? ( index%2==1 ? "#202020" : "transparent" ) : ( index%2==1 ? "#d0d0d0" : "transparent" ) )
            }
            Image {
                id: siteIcon
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: parent.left
                anchors.leftMargin: 3
                source: model.icon ? (testIcon.status == Image.Error ? QmlHelperTools.getFaviconFromUrl(model.url) : model.icon) : QmlHelperTools.getFaviconFromUrl(model.url)
                width: 20
                height: 20
                smooth: true
                cache: true
            }
            Image {
                id: testIcon
                visible: false
                source: model.icon
            }
            Text {
                id: siteLabel
                anchors.top: parent.top
                anchors.topMargin: 3
                anchors.left: siteIcon.right
                anchors.leftMargin: 10
                text: model.title
                font.pixelSize: 18
                color: theme.inverted ? "white" : "black"
            }
            Text {
                id: siteUrl
                anchors.top: siteLabel.bottom
                anchors.topMargin: 1
                anchors.left: siteIcon.right
                anchors.leftMargin: 10
                text: model.url
                font.underline: true
                font.pixelSize: 18
                color: theme.inverted ? "white" : "black"
            }
            MouseArea {
                id: mArea
                anchors.fill: parent
                onClicked: {
                    mainPage.load(model.url)
                    pageStack.pop()
                }
            }
        }
    }

    ScrollDecorator {
        flickableItem: recentList
    }

    ListView {
        id: recentSitesList
        anchors.top: address.bottom
        anchors.topMargin: 10
        anchors.left: parent.left
        anchors.right: parent.right
        height: 0
        clip: true
        visible: root.showRecent
        model: addressListModel
        delegate: Item {
            id: itemDelegate
            width: parent.width
            height: 50
            Rectangle {
                anchors.fill: parent
                color: mArea.pressed ? "#808080" : (theme.inverted ? "black" : "white")
            }
            Image {
                id: siteIcon
                anchors.verticalCenter: parent.verticalCenter
                anchors.left: parent.left
                anchors.leftMargin: 3
                source: model.icon ? (testIcon.status == Image.Error ? QmlHelperTools.getFaviconFromUrl(model.url) : model.icon) : QmlHelperTools.getFaviconFromUrl(model.url)
                width: 20
                height: 20
                smooth: true
                cache: true
            }
            Image {
                id: testIcon
                visible: false
                source: model.icon
            }
            Text {
                id: siteLabel
                anchors.top: parent.top
                anchors.topMargin: 3
                anchors.left: siteIcon.right
                anchors.leftMargin: 3
                text: model.title
                font.pixelSize: 18
                color: theme.inverted ? "white" : "black"
            }
            Text {
                id: siteUrl
                anchors.top: siteLabel.bottom
                anchors.topMargin: 1
                anchors.left: siteIcon.right
                anchors.leftMargin: 3
                text: model.url
                font.pixelSize: 18
                color: theme.inverted ? "white" : "black"
            }
            MouseArea {
                id: mArea
                anchors.fill: itemDelegate
                onClicked: {
                    hideRecentList()
                    mainPage.load(model.url)
                    mainPage.addressField.text = model.url
                    pageStack.pop()
                }
            }
        }
    }

    ScrollDecorator {
        flickableItem: recentSitesList
    }

    Rectangle {
        id: hideRecent
        anchors.top: recentSitesList.bottom
        width: parent.width
        height: visible ? 30 : 0
        visible: recentSitesList.visible
        color: theme.inverted ? "black" : "white"
        Rectangle {
            width: parent.width / 3 * 2
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.top: parent.top
            anchors.topMargin: 10
            height: 1
            color: theme.inverted? "white" : "black"
        }
        Rectangle {
            width: parent.width / 3 * 2
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.top: parent.top
            anchors.topMargin: 20
            height: 1
            color: theme.inverted? "white" : "black"
        }
        Rectangle {
            anchors.bottom: parent.bottom
            width: parent.width
            height: 1
            color: theme.inverted? "white" : "black"
        }
        MouseArea {
            anchors.fill:parent
            onClicked: {
                hideRecentList()
            }
        }
    }

    ListModel {
        id: addressListModel
    }

    Rectangle {
        anchors.bottom: toolsMenu.top
        width: parent.width
        height: 1
        color: theme.inverted ? "white" : "black"
    }

    ToolsMenu {
        id: toolsMenu
        anchors.bottom: parent.bottom
        width: (screen.currentOrientation == Screen.Portrait) ? parent.width : toolsMenu.toolsWidth
        anchors.left: parent.left
        anchors.leftMargin: (screen.currentOrientation == Screen.Portrait) ? 0 : ((parent.width - toolsMenu.toolsWidth) / 2)
        dockMode: false
    }

    Rectangle {
        visible: firstLoad
        anchors.fill: parent
        color: "black"
    }

    BusyIndicator {
        anchors.centerIn: parent
        visible: firstLoad
        running: visible
        platformStyle: BusyIndicatorStyle { size: "large" }
    }
} 
