import QtQuick 1.1
import com.nokia.meego 1.0

Rectangle {
    id: root
    height: flick.height + (dockMode ? flick.height/6 : 4)
    color: "transparent"
    property bool forceVisible: false
    property bool dockMode: true
    property int toolsWidth: buttonsColumn.width

    Connections {
        target: mainPage.viewportMouseArea
        onPressed: handleMouse(mouse, false)
        onPositionChanged: handleMouse(mouse, false)
        onReleased: handleMouse(mouse, true)
    }

    Connections {
        target: mainPage.viewportChild
        onUrlChanged: {
            favorite.isBookmark = bookmarksPage.checkUrl(mainPage.viewportChild.url)
        }
    }

    Connections {
        target: bookmarksPage
        onBookmarksChanged: {
            favorite.isBookmark = bookmarksPage.checkUrl(mainPage.viewportChild.url)
        }
    }

    function handleMouse(mouse, isReleased) {
        if (root.visible) {
            var mapped = root.mapFromItem(mainScope, mouse.x, mouse.y)
            if (mapped.x < 0 || mapped.x > root.width || mapped.y < 0 || mapped.y > root.height) {
                root.anchors.bottomMargin = 0 - root.height / 3 * 2
            }
            else {
                if (isReleased) {
                    root.forceVisible = true
                }
                else {
                    root.anchors.bottomMargin = 0
                }
            }
        }
    }

    Rectangle {
        id: dock
        anchors.bottom: parent.bottom
        width: parent.width
        height: root.height / 3
        color: theme.inverted ? "black" : "white"
        visible: root.dockMode
    }

    Rectangle {
        id: separator
        anchors.top: dock.top
        width: parent.width
        height: 1
        color: theme.inverted ? "white" : "black"
        visible: root.dockMode
    }

    Flickable {
        id: flick
        clip: true
        anchors.bottom: parent.bottom
        anchors.bottomMargin: root.dockMode ? height / 6 : 2
        height: buttonsColumn.height
        width: parent.width
        flickableDirection: Flickable.HorizontalFlick
        contentWidth: buttonsColumn.width
        interactive: contentWidth > width

        Row {
            id: buttonsColumn
            height: newPage.height
            spacing: 10

            Button {
                id: goStart

                width: 100
                height: 100

                iconSource: mainPage.iconset + "/" + (mainPage.pageStack.currentPage == startPage ? "back" : "home") + ".png"

                onClicked: {
                    if (pageStack.currentPage == startPage) {
                        pageStack.pop()
                    }
                    else {
                        startPage.show()
                        pageStack.push(startPage)
                    }
                    root.forceVisible = false
                }
            }

            Button {
                id: newPage

                width: 100
                height: 100

                iconSource: mainPage.iconset + "/plus.png"

                onClicked: {
                    MozContext.newWindow("about:blank", 0)
                    root.forceVisible = false
                }
            }

            Button {
                id: favorite
                width: 100
                height: 100
                visible: !startPage.visible
                property bool isBookmark: bookmarksPage.checkUrl(mainPage.viewportChild.url)
                iconSource: mainPage.iconset + "/" + (favorite.isBookmark ? "" : "un") +"favorite.png"

                onClicked: {
                    if (favorite.isBookmark) {
                        removeBookmark(mainPage.viewportChild.url)
                    }
                    else {
                        addBookmark(mainPage.viewportChild.url, mainPage.viewportChild.title, "uncategorized", 0)
                    }
                    favorite.isBookmark = bookmarksPage.checkUrl(mainPage.viewportChild.url)
                }
            }

            Button {
                id: settings

                width: 100
                height: 100

                iconSource: mainPage.iconset + "/settings.png"

                onClicked: {
                    settingsPage.show()
                    pageStack.push(settingsPage)
                    root.forceVisible = false
                }
            }

            Button {
                id: history

                width: 100
                height: 100

                iconSource: mainPage.iconset + "/history.png"

                onClicked: {
                    historyPage.fillModelFromDatabase(0)
                    pageStack.push(historyPage)
                    root.forceVisible = false
                }
            }

            Button {
                id: bookmarks

                width: 100
                height: 100

                iconSource: mainPage.iconset + "/bookmarks.png"

                onClicked: {
                    bookmarksPage.fillModelFromDatabase()
                    bookmarksPage.fillGroupsModel()
                    pageStack.push(bookmarksPage)
                    root.forceVisible = false
                }
            }

            Button {
                id: downloads

                width: 100
                height: 100

                iconSource: mainPage.iconset + "/download.png"

                onClicked: {                    
                    MozContext.sendObserve("embedui:download", { msg: "requestDownloadsList" })
                    pageStack.push(downloadsPage)
                    root.forceVisible = false
                }
            }
        }
    }
} 
