import QtQuick 1.1
import com.nokia.meego 1.0

Page {
    id: root
    orientationLock: PageOrientation.Automatic
    property int countTaps: 0

    Item {
        anchors.fill: parent

        AxisTestItem {
            id: axisTest
            anchors.fill: parent
            simple: true
            active: true
            onTap: {
                root.countTaps++

                if (root.countTaps > 4) {
                    axisTest.newAnchorY = axisTest.totalOffsetY / root.countTaps
                    mainPage.touchPointOffsetX = 0
                    mainPage.touchPointOffsetY = axisTest.newAnchorY
                    settingsPage.saveSetting("touch_offset_x", "" + 0)
                    settingsPage.saveSetting("touch_offset_y", "" + axisTest.newAnchorY)
                }
            }
        }

        Text {
            id: infoText
            height: 22
            font.pixelSize: height
            width: parent.width
            anchors.top: parent.top
            anchors.topMargin: 10
            horizontalAlignment: Text.AlignHCenter
            color: theme.inverted ? "white" : "black"
            text: root.countTaps < 5 ? qsTrId("meefox_test_more") : qsTrId("meefox_test_fixed")
        }

        Text {
            id: offsetText
            height: 22
            font.pixelSize: height
            width: parent.width
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 10
            horizontalAlignment: Text.AlignHCenter
            color: theme.inverted ? "white" : "black"
            text: root.countTaps > 4 ? qsTrId("meefox_test_offset") + ": " + axisTest.newAnchorY : ""
        }

        Button {
            id: resetButton
            anchors.right: parent.right
            anchors.rightMargin: 10
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 10
            width: 100
            text: qsTrId("meefox_test_reset")
            visible: root.countTaps > 4
            onClicked: {
                root.countTaps = 0
                axisTest.offsetY = 0
                axisTest.newAnchorY = 0
                axisTest.totalOffsetY = 0
                mainPage.touchPointOffsetX = 0
                mainPage.touchPointOffsetY = 0
            }
        }

        Button {
            id: backButton
            anchors.left: parent.left
            anchors.leftMargin: 10
            anchors.bottom: parent.bottom
            anchors.bottomMargin: 10
            width: 100
            text: qsTrId("meefox_common_back")
            onClicked: {
                pageStack.pop()
                var db = openDatabaseSync("qmlbrowser","0.1","historydb", 100000)
                db.transaction(
                    function(tx) {
                        tx.executeSql('INSERT INTO settings VALUES (?,?);',["touch_offset_x", mainPage.touchPointOffsetX])
                        tx.executeSql('INSERT INTO settings VALUES (?,?);',["touch_offset_y", mainPage.touchPointOffsetY])
                    }
                );
            }
        }
    }
} 
